@@ -13,7 +13,7 @@ if is_service_enabled magnum-api magnum-cond; then
         echo_summary "Installing magnum"
         install_magnum
 
-        MAGNUM_GUEST_IMAGE_URL=${MAGNUM_GUEST_IMAGE_URL:-"https://fedorapeople.org/groups/magnum/fedora-atomic-newton.qcow2"}
+        MAGNUM_GUEST_IMAGE_URL=${MAGNUM_GUEST_IMAGE_URL:-"https://builds.coreos.fedoraproject.org/prod/streams/stable/builds/31.20200323.3.2/x86_64/fedora-coreos-31.20200323.3.2-openstack.x86_64.qcow2.xz"}
         IMAGE_URLS+=",${MAGNUM_GUEST_IMAGE_URL}"
 
         LIBS_FROM_GIT="${LIBS_FROM_GIT},python-magnumclient"
@@ -35,12 +35,14 @@ if is_service_enabled magnum-api magnum-cond; then
         # Initialize magnum
         init_magnum
         magnum_register_image
+        magnum_configure_flavor
 
         # Start the magnum API and magnum taskmgr components
         echo_summary "Starting magnum"
         start_magnum
 
-        configure_iptables
+        configure_iptables_magnum
+        configure_apache_magnum
     fi
 
     if [[ "$1" == "unstack" ]]; then
