@@ -30,8 +30,9 @@ extensions = [
 ]
 
 # openstackdocstheme options
-repository_name = 'openstack/magnum'
-use_storyboard = True
+openstackdocs_repo_name = 'openstack/magnum'
+openstackdocs_pdf_link = True
+openstackdocs_use_storyboard = True
 
 config_generator_config_file = '../../etc/magnum/magnum-config-generator.conf'
 sample_config_basename = '_static/magnum'
@@ -60,7 +61,7 @@ add_function_parentheses = True
 add_module_names = True
 
 # The name of the Pygments (syntax highlighting) style to use.
-pygments_style = 'sphinx'
+pygments_style = 'native'
 
 # -- Options for HTML output --------------------------------------------------
 
@@ -78,10 +79,23 @@ htmlhelp_basename = 'magnumdoc'
 # [howto/manual]).
 latex_documents = [
     ('index',
-     'magnum.tex',
+     'doc-magnum.tex',
      u'magnum Documentation',
      u'OpenStack Foundation', 'manual'),
 ]
 
+# If false, no module index is generated.
+latex_domain_indices = False
+
+latex_elements = {
+    'makeindex': '',
+    'printindex': '',
+    'preamble': r'\setcounter{tocdepth}{3}',
+    'maxlistdepth': 10,
+}
+
+# Disable usage of xindy https://bugzilla.redhat.com/show_bug.cgi?id=1643664
+latex_use_xindy = False
+
 # Example configuration for intersphinx: refer to the Python standard library.
 # intersphinx_mapping = {'http://docs.python.org/': None}
