@@ -18,6 +18,7 @@ import uuid
 from oslo_log import log as logging
 from oslo_utils import timeutils
 import pecan
+import six
 import wsme
 from wsme import types as wtypes
 
@@ -33,7 +34,6 @@ from magnum.common import clients
 from magnum.common import exception
 from magnum.common import name_generator
 from magnum.common import policy
-from magnum.i18n import _LW
 from magnum import objects
 from magnum.objects import fields
 
@@ -90,6 +90,21 @@ class Bay(base.APIBase):
     master_count = wsme.wsattr(wtypes.IntegerType(minimum=1), default=1)
     """The number of master nodes for this bay. Default to 1 if not set"""
 
+    docker_volume_size = wtypes.IntegerType(minimum=1)
+    """The size in GB of the docker volume"""
+
+    labels = wtypes.DictType(wtypes.text, types.MultiType(wtypes.text,
+                                                          six.integer_types,
+                                                          bool,
+                                                          float))
+    """One or more key/value pairs"""
+
+    master_flavor_id = wtypes.StringType(min_length=1, max_length=255)
+    """The master flavor of this Bay"""
+
+    flavor_id = wtypes.StringType(min_length=1, max_length=255)
+    """The flavor of this Bay"""
+
     bay_create_timeout = wsme.wsattr(wtypes.IntegerType(minimum=0), default=60)
     """Timeout for creating the bay in minutes. Default to 60 if not set"""
 
@@ -99,7 +114,7 @@ class Bay(base.APIBase):
     stack_id = wsme.wsattr(wtypes.text, readonly=True)
     """Stack id of the heat stack"""
 
-    status = wtypes.Enum(str, *fields.ClusterStatus.ALL)
+    status = wtypes.Enum(wtypes.text, *fields.ClusterStatus.ALL)
     """Status of the bay from the heat stack"""
 
     status_reason = wtypes.text
@@ -124,9 +139,23 @@ class Bay(base.APIBase):
     master_addresses = wsme.wsattr([wtypes.text], readonly=True)
     """IP addresses of cluster master nodes"""
 
-    bay_faults = wsme.wsattr(wtypes.DictType(str, wtypes.text))
+    bay_faults = wsme.wsattr(wtypes.DictType(wtypes.text, wtypes.text))
     """Fault info collected from the heat resources of this bay"""
 
+    fixed_network = wtypes.StringType(min_length=1, max_length=255)
+    """The fixed network name to attach to the Cluster"""
+
+    fixed_subnet = wtypes.StringType(min_length=1, max_length=255)
+    """The fixed subnet name to attach to the Cluster"""
+
+    floating_ip_enabled = wsme.wsattr(types.boolean, default=True)
+    """Indicates whether created clusters should have a floating ip or not."""
+
+    master_lb_enabled = wsme.wsattr(types.boolean)
+    """Indicates whether created clusters should have a load balancer for master
+       nodes or not.
+       """
+
     def __init__(self, **kwargs):
         super(Bay, self).__init__()
 
@@ -168,10 +197,18 @@ class Bay(base.APIBase):
         else:
             setattr(self, 'bay_faults', kwargs.get('faults', wtypes.Unset))
 
+        nodegroup_fields = ['node_count', 'master_count',
+                            'node_addresses', 'master_addresses']
+        for field in nodegroup_fields:
+            self.fields.append(field)
+            setattr(self, field, kwargs.get(field, wtypes.Unset))
+
     @staticmethod
     def _convert_with_links(bay, url, expand=True):
         if not expand:
             bay.unset_fields_except(['uuid', 'name', 'baymodel_id',
+                                     'docker_volume_size', 'labels',
+                                     'master_flavor_id', 'flavor_id',
                                      'node_count', 'status',
                                      'bay_create_timeout', 'master_count',
                                      'stack_id'])
@@ -195,6 +232,10 @@ class Bay(base.APIBase):
                      baymodel_id='4a96ac4b-2447-43f1-8ca6-9fd6f36d146d',
                      node_count=2,
                      master_count=1,
+                     docker_volume_size=1,
+                     labels={},
+                     master_flavor_id=None,
+                     flavor_id=None,
                      bay_create_timeout=15,
                      stack_id='49dc23f5-ffc9-40c3-9d34-7be7f9e34d63',
                      status=fields.ClusterStatus.CREATE_COMPLETE,
@@ -273,7 +314,7 @@ class BaysController(base.Controller):
     }
 
     def _generate_name_for_bay(self, context):
-        '''Generate a random name like: zeta-22-bay.'''
+        """Generate a random name like: zeta-22-bay."""
         name_gen = name_generator.NameGenerator()
         name = name_gen.generate()
         return name + '-bay'
@@ -355,9 +396,9 @@ class BaysController(base.Controller):
                 bay.stack_id, nested_depth=2, filters=filters)
         except Exception as e:
             failed_resources = []
-            LOG.warning(_LW("Failed to retrieve failed resources for "
-                            "bay %(bay)s from Heat stack %(stack)s "
-                            "due to error: %(e)s"),
+            LOG.warning("Failed to retrieve failed resources for "
+                        "bay %(bay)s from Heat stack %(stack)s "
+                        "due to error: %(e)s",
                         {'bay': bay.uuid, 'stack': bay.stack_id, 'e': e},
                         exc_info=True)
 
@@ -372,7 +413,7 @@ class BaysController(base.Controller):
         """
         context = pecan.request.context
         bay = api_utils.get_resource('Cluster', bay_ident)
-        policy.enforce(context, 'bay:get', bay,
+        policy.enforce(context, 'bay:get', bay.as_dict(),
                        action='bay:get')
 
         bay = Bay.convert_with_links(bay)
@@ -389,8 +430,9 @@ class BaysController(base.Controller):
 
         :param bay: a bay within the request body.
         """
-        new_bay = self._post(bay)
+        new_bay, node_count, master_count = self._post(bay)
         res_bay = pecan.request.rpcapi.cluster_create(new_bay,
+                                                      master_count, node_count,
                                                       bay.bay_create_timeout)
 
         # Set the HTTP Location Header
@@ -399,13 +441,14 @@ class BaysController(base.Controller):
 
     @base.Controller.api_version("1.2")  # noqa
     @expose.expose(BayID, body=Bay, status_code=202)
-    def post(self, bay):
+    def post(self, bay):  # noqa
         """Create a new bay.
 
         :param bay: a bay within the request body.
         """
-        new_bay = self._post(bay)
+        new_bay, node_count, master_count = self._post(bay)
         pecan.request.rpcapi.cluster_create_async(new_bay,
+                                                  master_count, node_count,
                                                   bay.bay_create_timeout)
         return BayID(new_bay.uuid)
 
@@ -415,21 +458,43 @@ class BaysController(base.Controller):
                        action='bay:create')
         baymodel = objects.ClusterTemplate.get_by_uuid(context,
                                                        bay.baymodel_id)
-        attr_validator.validate_os_resources(context, baymodel.as_dict())
-        attr_validator.validate_master_count(bay.as_dict(), baymodel.as_dict())
+
+        # If docker_volume_size is not present, use baymodel value
+        if bay.docker_volume_size == wtypes.Unset:
+            bay.docker_volume_size = baymodel.docker_volume_size
+
+        # If labels is not present, use baymodel value
+        if bay.labels is None:
+            bay.labels = baymodel.labels
+
+        # If master_flavor_id is not present, use baymodel value
+        if bay.master_flavor_id == wtypes.Unset or not bay.master_flavor_id:
+            bay.master_flavor_id = baymodel.master_flavor_id
+
+        # If flavor_id is not present, use baymodel value
+        if bay.flavor_id == wtypes.Unset or not bay.flavor_id:
+            bay.flavor_id = baymodel.flavor_id
+
         bay_dict = bay.as_dict()
+        bay_dict['keypair'] = baymodel.keypair_id
+        attr_validator.validate_os_resources(context, baymodel.as_dict(),
+                                             bay_dict)
+        attr_validator.validate_master_count(bay.as_dict(), baymodel.as_dict())
+
         bay_dict['project_id'] = context.project_id
         bay_dict['user_id'] = context.user_id
         # NOTE(yuywz): We will generate a random human-readable name for
-        # bay if the name is not spcified by user.
+        # bay if the name is not specified by user.
         name = bay_dict.get('name') or self._generate_name_for_bay(context)
+        node_count = bay_dict.pop('node_count')
+        master_count = bay_dict.pop('master_count')
         bay_dict['name'] = name
         bay_dict['coe_version'] = None
         bay_dict['container_version'] = None
 
         new_bay = objects.Cluster(context, **bay_dict)
         new_bay.uuid = uuid.uuid4()
-        return new_bay
+        return new_bay, node_count, master_count
 
     @base.Controller.api_version("1.1", "1.1")
     @wsme.validate(types.uuid, [BayPatchType])
@@ -440,66 +505,72 @@ class BaysController(base.Controller):
         :param bay_ident: UUID or logical name of a bay.
         :param patch: a json PATCH document to apply to this bay.
         """
-        bay = self._patch(bay_ident, patch)
-        res_bay = pecan.request.rpcapi.cluster_update(bay)
+        bay, node_count = self._patch(bay_ident, patch)
+        res_bay = pecan.request.rpcapi.cluster_update(bay, node_count)
         return Bay.convert_with_links(res_bay)
 
     @base.Controller.api_version("1.2", "1.2")   # noqa
     @wsme.validate(types.uuid, [BayPatchType])
     @expose.expose(BayID, types.uuid_or_name, body=[BayPatchType],
                    status_code=202)
-    def patch(self, bay_ident, patch):
+    def patch(self, bay_ident, patch):  # noqa
         """Update an existing bay.
 
         :param bay_ident: UUID or logical name of a bay.
         :param patch: a json PATCH document to apply to this bay.
         """
-        bay = self._patch(bay_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(bay)
+        bay, node_count = self._patch(bay_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(bay, node_count)
         return BayID(bay.uuid)
 
     @base.Controller.api_version("1.3")   # noqa
     @wsme.validate(types.uuid, bool, [BayPatchType])
-    @expose.expose(BayID, types.uuid_or_name, bool, body=[BayPatchType],
-                   status_code=202)
-    def patch(self, bay_ident, rollback=False, patch=None):
+    @expose.expose(BayID, types.uuid_or_name, types.boolean,
+                   body=[BayPatchType], status_code=202)
+    def patch(self, bay_ident, rollback=False, patch=None):  # noqa
         """Update an existing bay.
 
         :param bay_ident: UUID or logical name of a bay.
         :param rollback: whether to rollback bay on update failure.
         :param patch: a json PATCH document to apply to this bay.
         """
-        bay = self._patch(bay_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(bay, rollback=rollback)
+        bay, node_count = self._patch(bay_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(bay, node_count,
+                                                  rollback=rollback)
         return BayID(bay.uuid)
 
     def _patch(self, bay_ident, patch):
         context = pecan.request.context
         bay = api_utils.get_resource('Cluster', bay_ident)
-        policy.enforce(context, 'bay:update', bay,
+        policy.enforce(context, 'bay:update', bay.as_dict(),
                        action='bay:update')
+
+        bay_to_cluster_attrs = {
+            'baymodel_id': 'cluster_template_id',
+            'bay_create_timeout': 'create_timeout'
+        }
         try:
             bay_dict = bay.as_dict()
             new_bay = Bay(**api_utils.apply_jsonpatch(bay_dict, patch))
         except api_utils.JSONPATCH_EXCEPTIONS as e:
             raise exception.PatchError(patch=patch, reason=e)
 
-        # Update only the fields that have changed
-        for field in objects.Cluster.fields:
-            try:
-                patch_val = getattr(new_bay, field)
-            except AttributeError:
-                # Ignore fields that aren't exposed in the API
+        # NOTE(ttsiouts): magnum.objects.Cluster.node_count will be a
+        # property so we won't be able to store it in the object. So
+        # instead of object_what_changed compare the new and the old
+        # clusters.
+        delta = set()
+        for field in new_bay.fields:
+            cluster_field = field
+            if cluster_field in bay_to_cluster_attrs:
+                cluster_field = bay_to_cluster_attrs[field]
+            if cluster_field not in bay_dict:
                 continue
-            if patch_val == wtypes.Unset:
-                patch_val = None
-            if bay[field] != patch_val:
-                bay[field] = patch_val
-
-        delta = bay.obj_what_changed()
+            if getattr(new_bay, field) != bay_dict[cluster_field]:
+                delta.add(cluster_field)
 
         validate_cluster_properties(delta)
-        return bay
+        return bay, new_bay.node_count
 
     @base.Controller.api_version("1.1", "1.1")
     @expose.expose(None, types.uuid_or_name, status_code=204)
@@ -514,7 +585,7 @@ class BaysController(base.Controller):
 
     @base.Controller.api_version("1.2")  # noqa
     @expose.expose(None, types.uuid_or_name, status_code=204)
-    def delete(self, bay_ident):
+    def delete(self, bay_ident):   # noqa
         """Delete a bay.
 
         :param bay_ident: UUID of a bay or logical name of the bay.
@@ -526,6 +597,6 @@ class BaysController(base.Controller):
     def _delete(self, bay_ident):
         context = pecan.request.context
         bay = api_utils.get_resource('Cluster', bay_ident)
-        policy.enforce(context, 'bay:delete', bay,
+        policy.enforce(context, 'bay:delete', bay.as_dict(),
                        action='bay:delete')
         return bay
