@@ -16,28 +16,34 @@
 import uuid
 
 from oslo_log import log as logging
+from oslo_utils import strutils
 from oslo_utils import timeutils
 import pecan
+import six
+import warnings
 import wsme
 from wsme import types as wtypes
 
 from magnum.api import attr_validator
 from magnum.api.controllers import base
 from magnum.api.controllers import link
+from magnum.api.controllers.v1 import cluster_actions
 from magnum.api.controllers.v1 import collection
+from magnum.api.controllers.v1 import nodegroup
 from magnum.api.controllers.v1 import types
 from magnum.api import expose
 from magnum.api import utils as api_utils
-from magnum.api.validation import validate_cluster_properties
-from magnum.common import clients
+from magnum.api import validation
 from magnum.common import exception
 from magnum.common import name_generator
 from magnum.common import policy
-from magnum.i18n import _LW
+import magnum.conf
+from magnum.i18n import _
 from magnum import objects
 from magnum.objects import fields
 
 LOG = logging.getLogger(__name__)
+CONF = magnum.conf.CONF
 
 
 class ClusterID(wtypes.Base):
@@ -95,12 +101,31 @@ class Cluster(base.APIBase):
                                           mandatory=True)
     """The cluster_template UUID"""
 
+    keypair = wsme.wsattr(wtypes.StringType(min_length=1, max_length=255),
+                          default=None)
+    """The name of the nova ssh keypair"""
+
     node_count = wsme.wsattr(wtypes.IntegerType(minimum=1), default=1)
     """The node count for this cluster. Default to 1 if not set"""
 
     master_count = wsme.wsattr(wtypes.IntegerType(minimum=1), default=1)
     """The number of master nodes for this cluster. Default to 1 if not set"""
 
+    docker_volume_size = wtypes.IntegerType(minimum=1)
+    """The size in GB of the docker volume"""
+
+    labels = wtypes.DictType(wtypes.text, types.MultiType(wtypes.text,
+                                                          six.integer_types,
+                                                          bool,
+                                                          float))
+    """One or more key/value pairs"""
+
+    master_flavor_id = wtypes.StringType(min_length=1, max_length=255)
+    """The flavor of the master node for this Cluster"""
+
+    flavor_id = wtypes.StringType(min_length=1, max_length=255)
+    """The flavor of this Cluster"""
+
     create_timeout = wsme.wsattr(wtypes.IntegerType(minimum=0), default=60)
     """Timeout for creating the cluster in minutes. Default to 60 if not set"""
 
@@ -110,12 +135,18 @@ class Cluster(base.APIBase):
     stack_id = wsme.wsattr(wtypes.text, readonly=True)
     """Stack id of the heat stack"""
 
-    status = wtypes.Enum(str, *fields.ClusterStatus.ALL)
+    status = wtypes.Enum(wtypes.text, *fields.ClusterStatus.ALL)
     """Status of the cluster from the heat stack"""
 
     status_reason = wtypes.text
     """Status reason of the cluster from the heat stack"""
 
+    health_status = wtypes.Enum(wtypes.text, *fields.ClusterHealthStatus.ALL)
+    """Health status of the cluster from the native COE API"""
+
+    health_status_reason = wtypes.DictType(wtypes.text, wtypes.text)
+    """Health status reason of the cluster from the native COE API"""
+
     discovery_url = wtypes.text
     """Url used for cluster node discovery"""
 
@@ -129,15 +160,53 @@ class Cluster(base.APIBase):
     container_version = wsme.wsattr(wtypes.text, readonly=True)
     """Version of the container software. Example: docker version."""
 
+    project_id = wsme.wsattr(wtypes.text, readonly=True)
+    """Project id of the cluster belongs to"""
+
+    user_id = wsme.wsattr(wtypes.text, readonly=True)
+    """User id of the cluster belongs to"""
+
     node_addresses = wsme.wsattr([wtypes.text], readonly=True)
     """IP addresses of cluster slave nodes"""
 
     master_addresses = wsme.wsattr([wtypes.text], readonly=True)
     """IP addresses of cluster master nodes"""
 
-    faults = wsme.wsattr(wtypes.DictType(str, wtypes.text))
+    faults = wsme.wsattr(wtypes.DictType(wtypes.text, wtypes.text))
     """Fault info collected from the heat resources of this cluster"""
 
+    fixed_network = wtypes.StringType(min_length=1, max_length=255)
+    """The fixed network name to attach to the Cluster"""
+
+    fixed_subnet = wtypes.StringType(min_length=1, max_length=255)
+    """The fixed subnet name to attach to the Cluster"""
+
+    floating_ip_enabled = wsme.wsattr(types.boolean)
+    """Indicates whether created clusters should have a floating ip or not."""
+
+    merge_labels = wsme.wsattr(types.boolean, default=False)
+    """Indicates whether the labels will be merged with the CT labels."""
+
+    labels_overridden = wtypes.DictType(
+            wtypes.text, types.MultiType(
+                wtypes.text, six.integer_types, bool, float))
+    """Contains labels that have a value different than the parent labels."""
+
+    labels_added = wtypes.DictType(
+            wtypes.text, types.MultiType(
+                wtypes.text, six.integer_types, bool, float))
+    """Contains labels that do not exist in the parent."""
+
+    labels_skipped = wtypes.DictType(
+            wtypes.text, types.MultiType(
+                wtypes.text, six.integer_types, bool, float))
+    """Contains labels that exist in the parent but were not inherited."""
+
+    master_lb_enabled = wsme.wsattr(types.boolean)
+    """Indicates whether created clusters should have a load balancer for master
+       nodes or not.
+       """
+
     def __init__(self, **kwargs):
         super(Cluster, self).__init__()
         self.fields = []
@@ -147,14 +216,27 @@ class Cluster(base.APIBase):
                 continue
             self.fields.append(field)
             setattr(self, field, kwargs.get(field, wtypes.Unset))
+        nodegroup_fields = ['node_count', 'master_count',
+                            'node_addresses', 'master_addresses']
+        for field in nodegroup_fields:
+            self.fields.append(field)
+            setattr(self, field, kwargs.get(field, wtypes.Unset))
 
     @staticmethod
-    def _convert_with_links(cluster, url, expand=True):
+    def _convert_with_links(cluster, url, expand=True, parent_labels=None):
         if not expand:
             cluster.unset_fields_except(['uuid', 'name', 'cluster_template_id',
-                                         'node_count', 'status',
+                                         'keypair', 'docker_volume_size',
+                                         'labels', 'node_count', 'status',
+                                         'master_flavor_id', 'flavor_id',
                                          'create_timeout', 'master_count',
-                                         'stack_id'])
+                                         'stack_id', 'health_status'])
+        else:
+            overridden, added, skipped = api_utils.get_labels_diff(
+                parent_labels, cluster.labels)
+            cluster.labels_overridden = overridden
+            cluster.labels_added = added
+            cluster.labels_skipped = skipped
 
         cluster.links = [link.Link.make_link('self', url,
                                              'clusters', cluster.uuid),
@@ -166,7 +248,9 @@ class Cluster(base.APIBase):
     @classmethod
     def convert_with_links(cls, rpc_cluster, expand=True):
         cluster = Cluster(**rpc_cluster.as_dict())
-        return cls._convert_with_links(cluster, pecan.request.host_url, expand)
+        parent_labels = rpc_cluster.cluster_template.labels
+        return cls._convert_with_links(cluster, pecan.request.host_url, expand,
+                                       parent_labels)
 
     @classmethod
     def sample(cls, expand=True):
@@ -174,18 +258,30 @@ class Cluster(base.APIBase):
         sample = cls(uuid='27e3153e-d5bf-4b7e-b517-fb518e17f34c',
                      name='example',
                      cluster_template_id=temp_id,
+                     keypair=None,
                      node_count=2,
                      master_count=1,
+                     docker_volume_size=1,
+                     labels={},
+                     master_flavor_id='m1.small',
+                     flavor_id='m1.small',
                      create_timeout=15,
                      stack_id='49dc23f5-ffc9-40c3-9d34-7be7f9e34d63',
                      status=fields.ClusterStatus.CREATE_COMPLETE,
                      status_reason="CREATE completed successfully",
+                     health_status=fields.ClusterHealthStatus.HEALTHY,
+                     health_status_reason={"api": "ok",
+                                           "node-0.Ready": 'True'},
                      api_address='172.24.4.3',
                      node_addresses=['172.24.4.4', '172.24.4.5'],
                      created_at=timeutils.utcnow(),
                      updated_at=timeutils.utcnow(),
                      coe_version=None,
-                     container_version=None)
+                     container_version=None,
+                     fixed_network=None,
+                     fixed_subnet=None,
+                     floating_ip_enabled=True,
+                     master_lb_enabled=True)
         return cls._convert_with_links(sample, 'http://localhost:9511', expand)
 
 
@@ -237,6 +333,14 @@ class ClustersController(base.Controller):
         'detail': ['GET'],
     }
 
+    _in_tree_cinder_volume_driver_deprecation_note = (
+        "The in-tree Cinder volume driver is deprecated and will be removed "
+        "in X cycle in favour of out-of-tree Cinder CSI driver which requires "
+        "the label cinder_csi_enabled set to True (default behaviour from "
+        "V cycle) when volume_driver is cinder.")
+
+    actions = cluster_actions.ActionsController()
+
     def _generate_name_for_cluster(self, context):
         """Generate a random name like: zeta-22-cluster."""
         name_gen = name_generator.NameGenerator()
@@ -247,6 +351,23 @@ class ClustersController(base.Controller):
                                  sort_key, sort_dir, expand=False,
                                  resource_url=None):
 
+        context = pecan.request.context
+        if context.is_admin:
+            if expand:
+                policy.enforce(context, "cluster:detail_all_projects",
+                               action="cluster:detail_all_projects")
+            else:
+                policy.enforce(context, "cluster:get_all_all_projects",
+                               action="cluster:get_all_all_projects")
+            # TODO(flwang): Instead of asking an extra 'all_project's
+            # parameter, currently the design is allowing admin user to list
+            # all clusters from all projects. But the all_tenants is one of
+            # the condition to do project filter in DB API. And it's also used
+            # by periodic tasks. So the could be removed in the future and
+            # a new parameter 'project_id' would be added so that admin user
+            # can list clusters for a particular project.
+            context.all_tenants = True
+
         limit = api_utils.validate_limit(limit)
         sort_dir = api_utils.validate_sort_dir(sort_dir)
 
@@ -265,6 +386,8 @@ class ClustersController(base.Controller):
                                                     sort_key=sort_key,
                                                     sort_dir=sort_dir)
 
+    nodegroups = nodegroup.NodeGroupController()
+
     @expose.expose(ClusterCollection, types.uuid, int, wtypes.text,
                    wtypes.text)
     def get_all(self, marker=None, limit=None, sort_key='id',
@@ -313,22 +436,11 @@ class ClustersController(base.Controller):
 
         and store them into cluster.faults.
         """
-        osc = clients.OpenStackClients(context)
-        filters = {'status': 'FAILED'}
-        try:
-            failed_resources = osc.heat().resources.list(
-                cluster.stack_id, nested_depth=2, filters=filters)
-        except Exception as e:
-            failed_resources = []
-            LOG.warning(_LW("Failed to retrieve failed resources for "
-                            "cluster %(cluster)s from Heat stack "
-                            "%(stack)s due to error: %(e)s"),
-                        {'cluster': cluster.uuid,
-                         'stack': cluster.stack_id, 'e': e},
-                        exc_info=True)
-
-        return {res.resource_name: res.resource_status_reason
-                for res in failed_resources}
+        # Gather fault info from the cluster nodegroups.
+        return {
+            ng.name: ng.status_reason for ng in cluster.nodegroups
+            if ng.status.endswith('FAILED')
+        }
 
     @expose.expose(Cluster, types.uuid_or_name)
     def get_one(self, cluster_ident):
@@ -337,18 +449,50 @@ class ClustersController(base.Controller):
         :param cluster_ident: UUID or logical name of the Cluster.
         """
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, "cluster:get_one_all_projects",
+                           action="cluster:get_one_all_projects")
+            # TODO(flwang): Instead of asking an extra 'all_project's
+            # parameter, currently the design is allowing admin user to list
+            # all clusters from all projects. But the all_tenants is one of
+            # the condition to do project filter in DB API. And it's also used
+            # by periodic tasks. So this could be removed in the future and
+            # a new parameter 'project_id' would be added so that admin user
+            # can list clusters for a particular project.
+            context.all_tenants = True
+
         cluster = api_utils.get_resource('Cluster', cluster_ident)
-        policy.enforce(context, 'cluster:get', cluster,
+        policy.enforce(context, 'cluster:get', cluster.as_dict(),
                        action='cluster:get')
 
-        cluster = Cluster.convert_with_links(cluster)
+        api_cluster = Cluster.convert_with_links(cluster)
 
-        if cluster.status in fields.ClusterStatus.STATUS_FAILED:
-            cluster.faults = self._collect_fault_info(context, cluster)
+        if api_cluster.status in fields.ClusterStatus.STATUS_FAILED:
+            api_cluster.faults = self._collect_fault_info(context, cluster)
 
-        return cluster
+        return api_cluster
+
+    def _check_cluster_quota_limit(self, context):
+        try:
+            # Check if there is any explicit quota limit set in Quotas table
+            quota = objects.Quota.get_quota_by_project_id_resource(
+                context,
+                context.project_id,
+                'Cluster')
+            cluster_limit = quota.hard_limit
+        except exception.QuotaNotFound:
+            # If explicit quota was not set for the project, use default limit
+            cluster_limit = CONF.quotas.max_clusters_per_project
+
+        if objects.Cluster.get_count_all(context) >= cluster_limit:
+            msg = _("You have reached the maximum clusters per project, "
+                    "%d. You may delete a cluster to make room for a new "
+                    "one.") % cluster_limit
+            raise exception.ResourceLimitExceeded(msg=msg)
 
     @expose.expose(ClusterID, body=Cluster, status_code=202)
+    @validation.enforce_cluster_type_supported()
+    @validation.enforce_cluster_volume_storage_size()
     def post(self, cluster):
         """Create a new cluster.
 
@@ -357,13 +501,54 @@ class ClustersController(base.Controller):
         context = pecan.request.context
         policy.enforce(context, 'cluster:create',
                        action='cluster:create')
+
+        self._check_cluster_quota_limit(context)
+
         temp_id = cluster.cluster_template_id
         cluster_template = objects.ClusterTemplate.get_by_uuid(context,
                                                                temp_id)
+        # If keypair not present, use cluster_template value
+        if cluster.keypair is None:
+            cluster.keypair = cluster_template.keypair_id
+
+        # If labels is not present, use cluster_template value
+        if cluster.labels == wtypes.Unset or not cluster.labels:
+            cluster.labels = cluster_template.labels
+        else:
+            # If labels are provided check if the user wishes to merge
+            # them with the values from the cluster template.
+            if cluster.merge_labels:
+                labels = cluster_template.labels
+                labels.update(cluster.labels)
+                cluster.labels = labels
+
+        cinder_csi_enabled = cluster.labels.get('cinder_csi_enabled', True)
+        if (cluster_template.volume_driver == 'cinder' and
+                not strutils.bool_from_string(cinder_csi_enabled)):
+            warnings.warn(self._in_tree_cinder_volume_driver_deprecation_note,
+                          DeprecationWarning)
+            LOG.warning(self._in_tree_cinder_volume_driver_deprecation_note)
+
+        # If floating_ip_enabled is not present, use cluster_template value
+        if cluster.floating_ip_enabled == wtypes.Unset:
+            cluster.floating_ip_enabled = cluster_template.floating_ip_enabled
+
+        # If master_lb_enabled is not present, use cluster_template value
+        if cluster.master_lb_enabled == wtypes.Unset:
+            cluster.master_lb_enabled = cluster_template.master_lb_enabled
+
+        attributes = ["docker_volume_size", "master_flavor_id", "flavor_id",
+                      "fixed_network", "fixed_subnet"]
+        for attr in attributes:
+            if (getattr(cluster, attr) == wtypes.Unset or
+                    not getattr(cluster, attr)):
+                setattr(cluster, attr, getattr(cluster_template, attr))
+
         cluster_dict = cluster.as_dict()
 
         attr_validator.validate_os_resources(context,
-                                             cluster_template.as_dict())
+                                             cluster_template.as_dict(),
+                                             cluster_dict)
         attr_validator.validate_master_count(cluster_dict,
                                              cluster_template.as_dict())
 
@@ -377,9 +562,12 @@ class ClustersController(base.Controller):
         cluster_dict['coe_version'] = None
         cluster_dict['container_version'] = None
 
+        node_count = cluster_dict.pop('node_count')
+        master_count = cluster_dict.pop('master_count')
         new_cluster = objects.Cluster(context, **cluster_dict)
         new_cluster.uuid = uuid.uuid4()
         pecan.request.rpcapi.cluster_create_async(new_cluster,
+                                                  master_count, node_count,
                                                   cluster.create_timeout)
 
         return ClusterID(new_cluster.uuid)
@@ -394,30 +582,46 @@ class ClustersController(base.Controller):
         :param cluster_ident: UUID or logical name of a cluster.
         :param patch: a json PATCH document to apply to this cluster.
         """
-        cluster = self._patch(cluster_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(cluster)
+        (cluster, node_count,
+         health_status,
+         health_status_reason) = self._patch(cluster_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(cluster, node_count,
+                                                  health_status,
+                                                  health_status_reason)
         return ClusterID(cluster.uuid)
 
     @base.Controller.api_version("1.3")  # noqa
     @wsme.validate(types.uuid, bool, [ClusterPatchType])
-    @expose.expose(ClusterID, types.uuid_or_name, bool,
+    @expose.expose(ClusterID, types.uuid_or_name, types.boolean,
                    body=[ClusterPatchType], status_code=202)
-    def patch(self, cluster_ident, rollback=False, patch=None):
+    def patch(self, cluster_ident, rollback=False, patch=None):  # noqa
         """Update an existing Cluster.
 
         :param cluster_ident: UUID or logical name of a cluster.
         :param rollback: whether to rollback cluster on update failure.
         :param patch: a json PATCH document to apply to this cluster.
         """
-        cluster = self._patch(cluster_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(cluster, rollback)
+        (cluster, node_count,
+         health_status,
+         health_status_reason) = self._patch(cluster_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(cluster, node_count,
+                                                  health_status,
+                                                  health_status_reason,
+                                                  rollback)
         return ClusterID(cluster.uuid)
 
     def _patch(self, cluster_ident, patch):
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, "cluster:update_all_projects",
+                           action="cluster:update_all_projects")
+            context.all_tenants = True
+
         cluster = api_utils.get_resource('Cluster', cluster_ident)
-        policy.enforce(context, 'cluster:update', cluster,
+        policy.enforce(context, 'cluster:update', cluster.as_dict(),
                        action='cluster:update')
+        policy.enforce(context, "cluster:update_health_status",
+                       action="cluster:update_health_status")
         try:
             cluster_dict = cluster.as_dict()
             new_cluster = Cluster(**api_utils.apply_jsonpatch(cluster_dict,
@@ -425,22 +629,28 @@ class ClustersController(base.Controller):
         except api_utils.JSONPATCH_EXCEPTIONS as e:
             raise exception.PatchError(patch=patch, reason=e)
 
-        # Update only the fields that have changed
-        for field in objects.Cluster.fields:
-            try:
-                patch_val = getattr(new_cluster, field)
-            except AttributeError:
-                # Ignore fields that aren't exposed in the API
-                continue
-            if patch_val == wtypes.Unset:
-                patch_val = None
-            if cluster[field] != patch_val:
-                cluster[field] = patch_val
+        # NOTE(ttsiouts): magnum.objects.Cluster.node_count will be a
+        # property so we won't be able to store it in the object. So
+        # instead of object_what_changed compare the new and the old
+        # clusters.
+        delta = set()
+        for field in new_cluster.fields:
+            if getattr(cluster, field) != getattr(new_cluster, field):
+                delta.add(field)
 
-        delta = cluster.obj_what_changed()
+        validation.validate_cluster_properties(delta)
 
-        validate_cluster_properties(delta)
-        return cluster
+        # NOTE(brtknr): cluster.node_count is the size of the whole cluster
+        # which includes non-default nodegroups. However cluster_update expects
+        # node_count to be the size of the default_ng_worker therefore return
+        # this value unless the patch object says otherwise.
+        node_count = cluster.default_ng_worker.node_count
+        for p in patch:
+            if p['path'] == '/node_count':
+                node_count = p.get('value') or new_cluster.node_count
+
+        return (cluster, node_count,
+                new_cluster.health_status, new_cluster.health_status_reason)
 
     @expose.expose(None, types.uuid_or_name, status_code=204)
     def delete(self, cluster_ident):
@@ -449,8 +659,13 @@ class ClustersController(base.Controller):
         :param cluster_ident: UUID of cluster or logical name of the cluster.
         """
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, 'cluster:delete_all_projects',
+                           action='cluster:delete_all_projects')
+            context.all_tenants = True
+
         cluster = api_utils.get_resource('Cluster', cluster_ident)
-        policy.enforce(context, 'cluster:delete', cluster,
+        policy.enforce(context, 'cluster:delete', cluster.as_dict(),
                        action='cluster:delete')
 
         pecan.request.rpcapi.cluster_delete_async(cluster.uuid)
