@@ -12,8 +12,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 from oslo_utils import timeutils
 import pecan
+import six
+import warnings
 import wsme
 from wsme import types as wtypes
 
@@ -32,6 +35,8 @@ from magnum.common import policy
 from magnum import objects
 from magnum.objects import fields
 
+LOG = logging.getLogger(__name__)
+
 
 class ClusterTemplate(base.APIBase):
     """API representation of a ClusterTemplate.
@@ -47,7 +52,7 @@ class ClusterTemplate(base.APIBase):
     name = wtypes.StringType(min_length=1, max_length=255)
     """The name of the ClusterTemplate"""
 
-    coe = wtypes.Enum(str, *fields.ClusterType.ALL, mandatory=True)
+    coe = wtypes.Enum(wtypes.text, *fields.ClusterType.ALL, mandatory=True)
     """The Container Orchestration Engine for this clustertemplate"""
 
     image_id = wsme.wsattr(wtypes.StringType(min_length=1, max_length=255),
@@ -60,7 +65,7 @@ class ClusterTemplate(base.APIBase):
     master_flavor_id = wtypes.StringType(min_length=1, max_length=255)
     """The flavor of the master node for this ClusterTemplate"""
 
-    dns_nameserver = wtypes.IPv4AddressType()
+    dns_nameserver = types.dns_list
     """The DNS nameserver address"""
 
     keypair_id = wsme.wsattr(wtypes.StringType(min_length=1, max_length=255),
@@ -112,7 +117,10 @@ class ClusterTemplate(base.APIBase):
     registry_enabled = wsme.wsattr(types.boolean, default=False)
     """Indicates whether the docker registry is enabled"""
 
-    labels = wtypes.DictType(str, str)
+    labels = wtypes.DictType(wtypes.text, types.MultiType(wtypes.text,
+                                                          six.integer_types,
+                                                          bool,
+                                                          float))
     """One or more key/value pairs"""
 
     tls_disabled = wsme.wsattr(types.boolean, default=False)
@@ -121,7 +129,7 @@ class ClusterTemplate(base.APIBase):
     public = wsme.wsattr(types.boolean, default=False)
     """Indicates whether the ClusterTemplate is public or not."""
 
-    server_type = wsme.wsattr(wtypes.Enum(str, *fields.ServerType.ALL),
+    server_type = wsme.wsattr(wtypes.Enum(wtypes.text, *fields.ServerType.ALL),
                               default='vm')
     """Server type for this ClusterTemplate """
 
@@ -145,6 +153,9 @@ class ClusterTemplate(base.APIBase):
     user_id = wsme.wsattr(wtypes.text, readonly=True)
     """User id of the cluster belongs to"""
 
+    hidden = wsme.wsattr(types.boolean, default=False)
+    """Indicates whether the ClusterTemplate is hidden or not."""
+
     def __init__(self, **kwargs):
         self.fields = []
         for field in objects.ClusterTemplate.fields:
@@ -201,7 +212,8 @@ class ClusterTemplate(base.APIBase):
             updated_at=timeutils.utcnow(),
             public=False,
             master_lb_enabled=False,
-            floating_ip_enabled=True)
+            floating_ip_enabled=True,
+            hidden=False)
         return cls._convert_with_links(sample, 'http://localhost:9511')
 
 
@@ -210,7 +222,7 @@ class ClusterTemplatePatchType(types.JsonPatchType):
     _extra_non_removable_attrs = {'/network_driver', '/external_network_id',
                                   '/tls_disabled', '/public', '/server_type',
                                   '/coe', '/registry_enabled',
-                                  '/cluster_distro'}
+                                  '/cluster_distro', '/hidden'}
 
 
 class ClusterTemplateCollection(collection.Collection):
@@ -244,6 +256,14 @@ class ClusterTemplatesController(base.Controller):
         'detail': ['GET'],
     }
 
+    _devicemapper_overlay_deprecation_note = (
+        "The devicemapper and overlay storage "
+        "drivers are deprecated in favor of overlay2 in docker, and will be "
+        "removed in a future release from docker. Users of the devicemapper "
+        "and overlay storage drivers are recommended to migrate to a "
+        "different storage driver, such as overlay2. overlay2 will be set "
+        "as the default storage driver from Victoria cycle in Magnum.")
+
     def _generate_name_for_cluster_template(self, context):
         """Generate a random name like: zeta-22-model."""
 
@@ -267,7 +287,7 @@ class ClusterTemplatesController(base.Controller):
             # parameter, currently the design is allowing admin user to list
             # all clusters from all projects. But the all_tenants is one of
             # the condition to do project filter in DB API. And it's also used
-            # by periodic tasks. So the could be removed in the future and
+            # by periodic tasks. So this could be removed in the future and
             # a new parameter 'project_id' would be added so that admin user
             # can list clusters for a particular project.
             context.all_tenants = True
@@ -348,7 +368,7 @@ class ClusterTemplatesController(base.Controller):
             # parameter, currently the design is allowing admin user to list
             # all clusters from all projects. But the all_tenants is one of
             # the condition to do project filter in DB API. And it's also used
-            # by periodic tasks. So the could be removed in the future and
+            # by periodic tasks. So this could be removed in the future and
             # a new parameter 'project_id' would be added so that admin user
             # can list clusters for a particular project.
             context.all_tenants = True
@@ -368,6 +388,7 @@ class ClusterTemplatesController(base.Controller):
     @validation.enforce_network_driver_types_create()
     @validation.enforce_volume_driver_types_create()
     @validation.enforce_volume_storage_size_create()
+    @validation.enforce_driver_supported()
     def post(self, cluster_template):
         """Create a new ClusterTemplate.
 
@@ -385,12 +406,18 @@ class ClusterTemplatesController(base.Controller):
         cluster_template_dict['cluster_distro'] = image_data['os_distro']
         cluster_template_dict['project_id'] = context.project_id
         cluster_template_dict['user_id'] = context.user_id
-        # check permissions for making cluster_template public
-        if cluster_template_dict['public']:
+        # check permissions for making cluster_template public or hidden
+        if cluster_template_dict['public'] or cluster_template_dict['hidden']:
             if not policy.enforce(context, "clustertemplate:publish", None,
                                   do_raise=False):
                 raise exception.ClusterTemplatePublishDenied()
 
+        if (cluster_template.docker_storage_driver in ('devicemapper',
+                                                       'overlay')):
+            warnings.warn(self._devicemapper_overlay_deprecation_note,
+                          DeprecationWarning)
+            LOG.warning(self._devicemapper_overlay_deprecation_note)
+
         # NOTE(yuywz): We will generate a random human-readable name for
         # cluster_template if the name is not specified by user.
         arg_name = cluster_template_dict.get('name')
@@ -405,12 +432,12 @@ class ClusterTemplatesController(base.Controller):
                                                  new_cluster_template.uuid)
         return ClusterTemplate.convert_with_links(new_cluster_template)
 
-    @wsme.validate(types.uuid_or_name, [ClusterTemplatePatchType])
+    @wsme.validate(types.uuid_or_name, [ClusterTemplatePatchType])  # noqa
     @expose.expose(ClusterTemplate, types.uuid_or_name,
                    body=[ClusterTemplatePatchType])
     @validation.enforce_network_driver_types_update()
     @validation.enforce_volume_driver_types_update()
-    def patch(self, cluster_template_ident, patch):
+    def patch(self, cluster_template_ident, patch):   # noqa
         """Update an existing ClusterTemplate.
 
         :param cluster_template_ident: UUID or logic name of a
@@ -419,6 +446,10 @@ class ClusterTemplatesController(base.Controller):
         ClusterTemplate.
         """
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, 'clustertemplate:update_all_projects',
+                           action='clustertemplate:update_all_projects')
+            context.all_tenants = True
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
         policy.enforce(context, 'clustertemplate:update',
@@ -435,8 +466,9 @@ class ClusterTemplatesController(base.Controller):
         new_cluster_template_dict = new_cluster_template.as_dict()
         attr_validator.validate_os_resources(context,
                                              new_cluster_template_dict)
-        # check permissions when updating ClusterTemplate public flag
-        if cluster_template.public != new_cluster_template.public:
+        # check permissions when updating ClusterTemplate public or hidden flag
+        if (cluster_template.public != new_cluster_template.public or
+                cluster_template.hidden != new_cluster_template.hidden):
             if not policy.enforce(context, "clustertemplate:publish", None,
                                   do_raise=False):
                 raise exception.ClusterTemplatePublishDenied()
@@ -453,6 +485,12 @@ class ClusterTemplatesController(base.Controller):
             if cluster_template[field] != patch_val:
                 cluster_template[field] = patch_val
 
+        if (cluster_template.docker_storage_driver in ('devicemapper',
+                                                       'overlay')):
+            warnings.warn(self._devicemapper_overlay_deprecation_note,
+                          DeprecationWarning)
+            LOG.warning(self._devicemapper_overlay_deprecation_note)
+
         cluster_template.save()
         return ClusterTemplate.convert_with_links(cluster_template)
 
@@ -464,6 +502,11 @@ class ClusterTemplatesController(base.Controller):
          ClusterTemplate.
         """
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, 'clustertemplate:delete_all_projects',
+                           action='clustertemplate:delete_all_projects')
+            context.all_tenants = True
+
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
         policy.enforce(context, 'clustertemplate:delete',
