@@ -1,10 +1,20 @@
-#!/bin/sh
-
 step="enable-prometheus-monitoring"
 printf "Starting to run ${step}\n"
 
 . /etc/sysconfig/heat-params
 
+if [ ! -z "$HTTP_PROXY" ]; then
+    export HTTP_PROXY
+fi
+
+if [ ! -z "$HTTPS_PROXY" ]; then
+    export HTTPS_PROXY
+fi
+
+if [ ! -z "$NO_PROXY" ]; then
+    export NO_PROXY
+fi
+
 function writeFile {
     # $1 is filename
     # $2 is file content
@@ -338,7 +348,7 @@ spec:
           name: grafana
           env:
             - name: GF_SECURITY_ADMIN_PASSWORD
-              value: $ADMIN_PASSWD
+              value: ${GRAFANA_ADMIN_PASSWD}
             - name: GF_DASHBOARDS_JSON_ENABLED
               value: "true"
             - name: GF_DASHBOARDS_JSON_PATH
