@@ -1,5 +1,3 @@
-#!/bin/sh
-
 . /etc/sysconfig/heat-params
 
 ssh_cmd="ssh -F /srv/magnum/.ssh/config root@localhost"
@@ -16,8 +14,13 @@ while [ ! -f /etc/kubernetes/certs/ca.key ] && \
 done
 
 echo "starting services"
+if [ ${CONTAINER_RUNTIME} = "containerd"  ] ; then
+    container_runtime_service="containerd"
+else
+    container_runtime_service="docker"
+fi
 for action in enable restart; do
-    for service in etcd docker kube-apiserver kube-controller-manager kube-scheduler kubelet kube-proxy; do
+    for service in etcd ${container_runtime_service} kube-apiserver kube-controller-manager kube-scheduler kubelet kube-proxy; do
         echo "$action service $service"
         $ssh_cmd systemctl $action $service
     done
@@ -31,3 +34,11 @@ do
     echo "Trying to label master node with node-role.kubernetes.io/master=\"\""
     sleep 5s
 done
+
+if [ "$(echo $USE_PODMAN | tr '[:upper:]' '[:lower:]')" == "true" ]; then
+    KUBE_DIGEST=$($ssh_cmd podman image inspect ${CONTAINER_INFRA_PREFIX:-k8s.gcr.io/}hyperkube:${KUBE_TAG} --format "{{.Digest}}")
+    if [ -n "${KUBE_IMAGE_DIGEST}"  ] && [ "${KUBE_IMAGE_DIGEST}" != "${KUBE_DIGEST}" ]; then
+        printf "The sha256 ${KUBE_DIGEST} of current hyperkube image cannot match the given one: ${KUBE_IMAGE_DIGEST}."
+        exit 1
+    fi
+fi
