@@ -1,6 +1,3 @@
-# coding=utf-8
-#
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -19,8 +16,7 @@ from magnum.objects import base
 
 
 @base.MagnumObjectRegistry.register
-class Certificate(base.MagnumPersistentObject, base.MagnumObject,
-                  base.MagnumObjectDictCompat):
+class Certificate(base.MagnumPersistentObject, base.MagnumObject):
     # Version 1.0: Initial version
     # Version 1.1: Rename bay_uuid to cluster_uuid
     VERSION = '1.1'
