@@ -38,16 +38,16 @@ function create_test_data {
     elif [ "${coe}${special}" == 'k8s-ironic' ]; then
         local bm_flavor_id=$(openstack flavor show baremetal -f value -c id)
         die_if_not_set $LINENO bm_flavor_id "Failed to get id of baremetal flavor"
-
+        # NOTE(TheJulia): This issue was fixed in Feb 2018 as part of change
+        # Ifb9a49d4258a559cf2175d902e9424a3f98065c5. Commented out in Oct 2018.
         # NOTE(yuanying): Workaround fix for ironic issue
         # cf. https://bugs.launchpad.net/ironic/+bug/1596421
-        echo "alter table ironic.nodes modify instance_info LONGTEXT;" | mysql -uroot -p${MYSQL_PASSWORD} ironic
+        # echo "alter table ironic.nodes modify instance_info LONGTEXT;" | mysql -uroot -p${MYSQL_PASSWORD} ironic
         # NOTE(yuanying): Ironic instances need to connect to Internet
-        neutron subnet-update private-subnet --dns-nameserver 8.8.8.8
-
+        openstack subnet set private-subnet --dns-nameserver 8.8.8.8
         local container_format="ami"
     else
-        local image_name="atomic"
+        local image_name="fedora-coreos"
         local container_format="bare"
     fi
 
@@ -56,13 +56,19 @@ function create_test_data {
     # setting, it allows to perform testing on custom images.
     image_name=${MAGNUM_IMAGE_NAME:-$image_name}
 
-    export NIC_ID=$(neutron net-show public | awk '/ id /{print $4}')
+    export NIC_ID=$(openstack network show public -f value -c id)
 
     # We need to filter by container_format to get the appropriate
     # image. Specifically, when we provide kernel and ramdisk images
     # we need to select the 'ami' image. Otherwise, when we have
     # qcow2 images, the format is 'bare'.
-    export IMAGE_ID=$(glance --os-image-api-version 1 image-list | grep $container_format | grep -i $image_name | awk '{print $2}')
+    export IMAGE_ID=$(openstack image list --property container_format=$container_format | grep -i $image_name | awk '{print $2}')
+
+    #Get magnum_url
+    local magnum_api_ip=$(iniget /etc/magnum/magnum.conf api host)
+    local magnum_api_port=$(iniget /etc/magnum/magnum.conf api port)
+    local magnum_url="http://"$magnum_api_ip":"$magnum_api_port"/v1"
+    local keystone_auth_url=$(iniget /etc/magnum/magnum.conf keystone_authtoken www_authenticate_uri)
 
     # pass the appropriate variables via a config file
     CREDS_FILE=$MAGNUM_DIR/functional_creds.conf
@@ -70,15 +76,20 @@ function create_test_data {
 # Credentials for functional testing
 
 [auth]
-auth_url = $OS_AUTH_URL
-magnum_url = $BYPASS_URL
+auth_url = $keystone_auth_url
+magnum_url = $magnum_url
 username = $OS_USERNAME
-tenant_name = $OS_TENANT_NAME
+project_name = $OS_PROJECT_NAME
+project_domain_id = $OS_PROJECT_DOMAIN_ID
+user_domain_id = $OS_USER_DOMAIN_ID
 password = $OS_PASSWORD
-auth_version = v2
+auth_version = v3
+insecure = False
 [admin]
 user = $OS_USERNAME
-tenant = $OS_TENANT_NAME
+project_name = $OS_PROJECT_NAME
+project_domain_id = $OS_PROJECT_DOMAIN_ID
+user_domain_id = $OS_USER_DOMAIN_ID
 pass = $OS_PASSWORD
 region_name = $OS_REGION_NAME
 [magnum]
@@ -97,8 +108,10 @@ EOF
 
     # Create a keypair for use in the functional tests.
     echo_summary "Generate a key-pair"
-    ssh-keygen -t rsa -N "" -f ~/.ssh/id_rsa
-    nova keypair-add  --pub-key ~/.ssh/id_rsa.pub default
+    # ~/.ssh/id_rsa already exists in multinode setup, so generate
+    # key with different name
+    ssh-keygen -t rsa -N "" -f ~/.ssh/id_rsa_magnum
+    openstack keypair create --public-key ~/.ssh/id_rsa_magnum.pub default
 }
 
 function add_flavor {
@@ -108,24 +121,18 @@ function add_flavor {
     # Get admin credentials
     pushd ../devstack
     source openrc admin admin
-    # NOTE(hongbin): This is a temporary work around. These variables are for
-    # keystone v3, but magnum is using v2 API. Therefore, unset them to make the
-    # keystoneclient work.
-    # Bug: #1473600
-    unset OS_PROJECT_DOMAIN_ID
-    unset OS_USER_DOMAIN_ID
-    unset OS_AUTH_TYPE
     popd
 
-    # Due to keystone defaulting everything to v3, we need to update to make func tests
-    # work in our gates back to v2
-    export OS_AUTH_URL=http://127.0.0.1:5000/v2.0
-    export OS_IDENTITY_API_VERSION=2.0
-
     # Create magnum specific flavor for use in functional tests.
     echo_summary "Create a flavor"
-    nova flavor-create  m1.magnum 100 1024 10 1
-    nova flavor-create  s1.magnum 200 512 10 1
+    if [[ "$DEVSTACK_GATE_TOPOLOGY" = "multinode" ]] ; then
+        local flavor_ram="3750"
+        local flavor_disk="20"
+        local flavor_vcpus="2"
+    fi
+
+    openstack flavor create m1.magnum --id 100 --ram ${flavor_ram:-1024} --disk ${flavor_disk:-10} --vcpus ${flavor_vcpus:-4}
+    openstack flavor create s1.magnum --id 200 --ram ${flavor_ram:-1024} --disk ${flavor_disk:-10} --vcpus ${flavor_vcpus:-4}
 }
 
 if ! function_exists echo_summary; then
@@ -148,7 +155,7 @@ constraints="-c $REQUIREMENTS_DIR/upper-constraints.txt"
 sudo -H pip install $constraints -U -r requirements.txt -r test-requirements.txt
 
 export MAGNUM_DIR="$BASE/new/magnum"
-sudo chown -R jenkins:stack $MAGNUM_DIR
+sudo chown -R $USER:stack $MAGNUM_DIR
 
 # Run functional tests
 # Currently we support functional-api, functional-k8s, will support swarm,
@@ -161,67 +168,33 @@ echo "Running magnum functional test suite for $1"
 coe=$1
 special=$2
 
-if [[ "api" == "$coe" ]]; then
-    # Import devstack functions 'iniset', 'iniget' and 'trueorfalse'
-    source $BASE/new/devstack/functions
-    echo "TEMPEST_SERVICES+=,magnum" >> $localrc_path
-    pushd $BASE/new/tempest
-    sudo chown -R jenkins:stack $BASE/new/tempest
-
+if [[ "-ironic" != "$special" ]]; then
     add_flavor
-
-    # Set demo credentials
-    source $BASE/new/devstack/accrc/demo/demo
-    unset OS_AUTH_TYPE
-
-    create_test_data $coe
-
-    # Set up tempest config with magnum goodness
-    iniset $BASE/new/tempest/etc/tempest.conf magnum image_id $IMAGE_ID
-    iniset $BASE/new/tempest/etc/tempest.conf magnum nic_id $NIC_ID
-    iniset $BASE/new/tempest/etc/tempest.conf magnum keypair_id default
-    iniset $BASE/new/tempest/etc/tempest.conf magnum flavor_id s1.magnum
-    iniset $BASE/new/tempest/etc/tempest.conf magnum master_flavor_id m1.magnum
-    iniset $BASE/new/tempest/etc/tempest.conf magnum copy_logs True
-
-    # show tempest config with magnum
-    cat etc/tempest.conf
-
-    # Set up concurrency and test regex
-    export MAGNUM_TEMPEST_CONCURRENCY=${MAGNUM_TEMPEST_CONCURRENCY:-1}
-    export MAGNUM_TESTS=${MAGNUM_TESTS:-'magnum.tests.functional.api.v1'}
-
-    echo "Running tempest magnum test suites"
-    sudo -H -u jenkins tox -eall-plugin -- $MAGNUM_TESTS --concurrency=$MAGNUM_TEMPEST_CONCURRENCY
-else
-    # Get admin credentials
-    pushd ../devstack
-    source openrc admin admin
-    # NOTE(hongbin): This is a temporary work around. These variables are for
-    # keystone v3, but magnum is using v2 API. Therefore, unset them to make the
-    # keystoneclient work.
-    # Bug: #1473600
-    unset OS_PROJECT_DOMAIN_ID
-    unset OS_USER_DOMAIN_ID
-    unset OS_AUTH_TYPE
-    popd
-
-    add_flavor
-
-    create_test_data $coe $special
-
-    target="${coe}${special}"
-    sudo -E -H -u jenkins tox -e functional-"$target" -- --concurrency=1
 fi
+
+# Get admin credentials
+pushd ../devstack
+source openrc admin admin
+popd
+
+create_test_data $coe $special
+
+_magnum_tests=""
+target="${coe}${special}"
+sudo -E -H -u $USER tox -e functional-"$target" $_magnum_tests -- --concurrency=1
 EXIT_CODE=$?
 
+
 # Delete the keypair used in the functional test.
 echo_summary "Running keypair-delete"
-nova keypair-delete default
+openstack keypair delete default
 
-# Delete the flavor used in the functional test.
-echo_summary "Running flavor-delete"
-nova flavor-delete m1.magnum
+if [[ "-ironic" != "$special" ]]; then
+    # Delete the flavor used in the functional test.
+    echo_summary "Running flavor-delete"
+    openstack flavor delete m1.magnum
+    openstack flavor delete s1.magnum
+fi
 
 # Save functional testing log
 sudo cp $MAGNUM_DIR/functional-tests.log /opt/stack/logs/
