@@ -10,8 +10,9 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+from unittest import mock
+
 import fixtures
-import mock
 from oslo_config import cfg
 from webob import exc as webob_exc
 
@@ -40,7 +41,7 @@ class TestRootController(api_base.FunctionalTest):
                                [{u'href': u'http://localhost/v1/',
                                  u'rel': u'self'}],
                            u'status': u'CURRENT',
-                           u'max_version': u'1.3',
+                           u'max_version': u'1.9',
                            u'min_version': u'1.1'}]}
 
         self.v1_expected = {
@@ -53,6 +54,10 @@ class TestRootController(api_base.FunctionalTest):
                         u'http://docs.openstack.org/developer'
                         '/magnum/dev/api-spec-v1.html',
                         u'type': u'text/html', u'rel': u'describedby'}],
+            u'stats': [{u'href': u'http://localhost/v1/stats/',
+                       u'rel': u'self'},
+                       {u'href': u'http://localhost/stats/',
+                        u'rel': u'bookmark'}],
             u'bays': [{u'href': u'http://localhost/v1/bays/',
                        u'rel': u'self'},
                       {u'href': u'http://localhost/bays/',
@@ -65,6 +70,10 @@ class TestRootController(api_base.FunctionalTest):
                            u'rel': u'self'},
                           {u'href': u'http://localhost/clusters/',
                            u'rel': u'bookmark'}],
+            u'quotas': [{u'href': u'http://localhost/v1/quotas/',
+                         u'rel': u'self'},
+                        {u'href': u'http://localhost/quotas/',
+                         u'rel': u'bookmark'}],
             u'clustertemplates':
                 [{u'href': u'http://localhost/v1/clustertemplates/',
                   u'rel': u'self'},
@@ -78,7 +87,17 @@ class TestRootController(api_base.FunctionalTest):
             u'mservices': [{u'href': u'http://localhost/v1/mservices/',
                             u'rel': u'self'},
                            {u'href': u'http://localhost/mservices/',
-                            u'rel': u'bookmark'}]}
+                            u'rel': u'bookmark'}],
+            u'federations': [{u'href': u'http://localhost/v1/federations/',
+                              u'rel': u'self'},
+                             {u'href': u'http://localhost/federations/',
+                              u'rel': u'bookmark'}],
+            u'nodegroups': [{u'href': u'http://localhost/v1/clusters/'
+                                      '{cluster_id}/nodegroups',
+                             u'rel': u'self'},
+                            {u'href': u'http://localhost/clusters/'
+                                      '{cluster_id}/nodegroups',
+                             u'rel': u'bookmark'}]}
 
     def make_app(self, paste_file):
         file_name = self.get_path(paste_file)
