@@ -11,8 +11,8 @@
 #    limitations under the License.
 
 import datetime
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
@@ -26,6 +26,7 @@ from magnum import objects
 from magnum.tests import base
 from magnum.tests.unit.api import base as api_base
 from magnum.tests.unit.api import utils as apiutils
+from magnum.tests.unit.db import utils as db_utils
 from magnum.tests.unit.objects import utils as obj_utils
 
 
@@ -203,9 +204,10 @@ class TestListBay(api_base.FunctionalTest):
         self.assertIn('links', response.keys())
         self.assertEqual(2, len(response['links']))
         self.assertIn(uuid, response['links'][0]['href'])
-        for l in response['links']:
-            bookmark = l['rel'] == 'bookmark'
-            self.assertTrue(self.validate_link(l['href'], bookmark=bookmark))
+        for link in response['links']:
+            bookmark = link['rel'] == 'bookmark'
+            self.assertTrue(self.validate_link(link['href'],
+                                               bookmark=bookmark))
 
     def test_collection_links(self):
         for id_ in range(5):
@@ -243,8 +245,12 @@ class TestPatch(api_base.FunctionalTest):
         self.mock_bay_update.side_effect = self._simulate_rpc_bay_update
         self.addCleanup(p.stop)
 
-    def _simulate_rpc_bay_update(self, bay, rollback=False):
+    def _simulate_rpc_bay_update(self, bay, node_count, rollback=False):
+        bay.status = 'UPDATE_IN_PROGRESS'
         bay.save()
+        default_ng_worker = bay.default_ng_worker
+        default_ng_worker.node_count = node_count
+        default_ng_worker.save()
         return bay
 
     @mock.patch('oslo_utils.timeutils.utcnow')
@@ -394,13 +400,15 @@ class TestPatch(api_base.FunctionalTest):
 
     @mock.patch.object(rpcapi.API, 'cluster_update_async')
     def test_update_bay_with_rollback_enabled(self, mock_update):
+        node_count = 4
         response = self.patch_json(
             '/bays/%s/?rollback=True' % self.bay.name,
-            [{'path': '/node_count', 'value': 4,
+            [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
-        mock_update.assert_called_once_with(mock.ANY, rollback=True)
+        mock_update.assert_called_once_with(mock.ANY, node_count,
+                                            rollback=True)
         self.assertEqual(202, response.status_code)
 
     def test_remove_ok(self):
@@ -455,8 +463,12 @@ class TestPost(api_base.FunctionalTest):
         self.mock_valid_os_res = p.start()
         self.addCleanup(p.stop)
 
-    def _simulate_rpc_bay_create(self, bay, bay_create_timeout):
+    def _simulate_rpc_bay_create(self, bay, master_count, node_count,
+                                 bay_create_timeout):
         bay.create()
+        db_utils.create_nodegroups_for_cluster(
+            cluster_id=bay.uuid, node_count=node_count,
+            master_count=master_count)
         return bay
 
     @mock.patch('oslo_utils.timeutils.utcnow')
@@ -481,10 +493,14 @@ class TestPost(api_base.FunctionalTest):
     def test_create_bay_set_project_id_and_user_id(self):
         bdict = apiutils.bay_post_data()
 
-        def _simulate_rpc_bay_create(bay, bay_create_timeout):
+        def _simulate_rpc_bay_create(bay, node_count, master_count,
+                                     bay_create_timeout):
             self.assertEqual(self.context.project_id, bay.project_id)
             self.assertEqual(self.context.user_id, bay.user_id)
             bay.create()
+            db_utils.create_nodegroups_for_cluster(
+                cluster_id=bay.uuid, node_count=node_count,
+                master_count=master_count)
             return bay
         self.mock_bay_create.side_effect = _simulate_rpc_bay_create
 
@@ -691,9 +707,13 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual(201, response.status_int)
 
     def test_create_bay_with_no_timeout(self):
-        def _simulate_rpc_bay_create(bay, bay_create_timeout):
+        def _simulate_rpc_bay_create(bay, node_count, master_count,
+                                     bay_create_timeout):
             self.assertEqual(60, bay_create_timeout)
             bay.create()
+            db_utils.create_nodegroups_for_cluster(
+                cluster_id=bay.uuid, node_count=node_count,
+                master_count=master_count)
             return bay
         self.mock_bay_create.side_effect = _simulate_rpc_bay_create
         bdict = apiutils.bay_post_data()
@@ -788,6 +808,26 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(400, response.status_int)
 
+    def test_create_bay_with_docker_volume_size(self):
+        bdict = apiutils.bay_post_data()
+        bdict['docker_volume_size'] = 3
+        response = self.post_json('/bays', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(201, response.status_int)
+        bay, timeout = self.mock_bay_create.call_args
+        self.assertEqual(3, bay[0].docker_volume_size)
+
+    def test_create_bay_without_docker_volume_size(self):
+        bdict = apiutils.bay_post_data()
+        # Remove the default docker_volume_size from the bay dict.
+        del bdict['docker_volume_size']
+        response = self.post_json('/bays', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(201, response.status_int)
+        bay, timeout = self.mock_bay_create.call_args
+        # Verify docker_volume_size from BayModel is used
+        self.assertEqual(20, bay[0].docker_volume_size)
+
 
 class TestDelete(api_base.FunctionalTest):
 
@@ -804,6 +844,9 @@ class TestDelete(api_base.FunctionalTest):
     def _simulate_rpc_bay_delete(self, bay_uuid):
         bay = objects.Cluster.get_by_uuid(self.context, bay_uuid)
         bay.destroy()
+        ngs = objects.NodeGroup.list(self.context, bay_uuid)
+        for ng in ngs:
+            ng.destroy()
 
     def test_delete_bay(self):
         self.delete('/bays/%s' % self.bay.uuid)
