@@ -11,9 +11,10 @@
 #    limitations under the License.
 
 import datetime
+from unittest import mock
 
-import mock
 from oslo_config import cfg
+from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from wsme import types as wtypes
@@ -27,6 +28,7 @@ from magnum import objects
 from magnum.tests import base
 from magnum.tests.unit.api import base as api_base
 from magnum.tests.unit.api import utils as apiutils
+from magnum.tests.unit.db import utils as db_utils
 from magnum.tests.unit.objects import utils as obj_utils
 
 CONF = magnum.conf.CONF
@@ -89,30 +91,15 @@ class TestListCluster(api_base.FunctionalTest):
         self.assertEqual(cluster.uuid, response['uuid'])
         self._verify_attrs(self._expand_cluster_attrs, response)
 
-    @mock.patch('magnum.common.clients.OpenStackClients.heat')
-    def test_get_one_failed_cluster(self, mock_heat):
-        fake_resources = mock.MagicMock()
-        fake_resources.resource_name = 'fake_name'
-        fake_resources.resource_status_reason = 'fake_reason'
-
-        ht = mock.MagicMock()
-        ht.resources.list.return_value = [fake_resources]
-        mock_heat.return_value = ht
-
+    def test_get_one_failed_cluster(self):
         cluster = obj_utils.create_test_cluster(self.context,
-                                                status='CREATE_FAILED')
+                                                status='CREATE_FAILED',
+                                                master_status='CREATE_FAILED',
+                                                master_reason='fake_reason')
         response = self.get_json('/clusters/%s' % cluster['uuid'])
+        expected_faults = {cluster.default_ng_master.name: 'fake_reason'}
         self.assertEqual(cluster.uuid, response['uuid'])
-        self.assertEqual({'fake_name': 'fake_reason'}, response['faults'])
-
-    @mock.patch('magnum.common.clients.OpenStackClients.heat')
-    def test_get_one_failed_cluster_heatclient_exception(self, mock_heat):
-        mock_heat.resources.list.side_effect = Exception('fake')
-        cluster = obj_utils.create_test_cluster(self.context,
-                                                status='CREATE_FAILED')
-        response = self.get_json('/clusters/%s' % cluster['uuid'])
-        self.assertEqual(cluster.uuid, response['uuid'])
-        self.assertEqual({}, response['faults'])
+        self.assertEqual(expected_faults, response['faults'])
 
     def test_get_one_by_name(self):
         cluster = obj_utils.create_test_cluster(self.context)
@@ -131,9 +118,27 @@ class TestListCluster(api_base.FunctionalTest):
     def test_get_one_by_uuid(self):
         temp_uuid = uuidutils.generate_uuid()
         obj_utils.create_test_cluster(self.context, uuid=temp_uuid)
-        response = self.get_json(
-            '/clusters/%s' % temp_uuid)
+        response = self.get_json('/clusters/%s' % temp_uuid)
         self.assertEqual(temp_uuid, response['uuid'])
+        self.assertIn('labels_overridden', response)
+        self.assertIn('labels_added', response)
+        self.assertIn('labels_skipped', response)
+
+    def test_get_one_merged_labels(self):
+        ct_uuid = uuidutils.generate_uuid()
+        ct_labels = {'label1': 'value1', 'label2': 'value2'}
+        obj_utils.create_test_cluster_template(self.context, uuid=ct_uuid,
+                                               labels=ct_labels)
+        c_uuid = uuidutils.generate_uuid()
+        c_labels = {'label1': 'value3', 'label4': 'value4'}
+        obj_utils.create_test_cluster(self.context, uuid=c_uuid,
+                                      labels=c_labels,
+                                      cluster_template_id=ct_uuid)
+        response = self.get_json('/clusters/%s' % c_uuid)
+        self.assertEqual(c_labels, response['labels'])
+        self.assertEqual({'label1': 'value1'}, response['labels_overridden'])
+        self.assertEqual({'label2': 'value2'}, response['labels_skipped'])
+        self.assertEqual({'label4': 'value4'}, response['labels_added'])
 
     def test_get_one_by_uuid_not_found(self):
         temp_uuid = uuidutils.generate_uuid()
@@ -241,9 +246,9 @@ class TestListCluster(api_base.FunctionalTest):
         self.assertIn('links', response.keys())
         self.assertEqual(2, len(response['links']))
         self.assertIn(uuid, response['links'][0]['href'])
-        for l in response['links']:
-            bookmark = l['rel'] == 'bookmark'
-            self.assertTrue(self.validate_link(l['href'],
+        for link in response['links']:
+            bookmark = link['rel'] == 'bookmark'
+            self.assertTrue(self.validate_link(link['href'],
                                                bookmark=bookmark))
 
     def test_collection_links(self):
@@ -274,13 +279,21 @@ class TestPatch(api_base.FunctionalTest):
         self.cluster_template_obj = obj_utils.create_test_cluster_template(
             self.context)
         self.cluster_obj = obj_utils.create_test_cluster(
-            self.context, name='cluster_example_A', node_count=3)
+            self.context, name='cluster_example_A', node_count=3,
+            health_status='UNKNOWN', health_status_reason={})
         p = mock.patch.object(rpcapi.API, 'cluster_update_async')
         self.mock_cluster_update = p.start()
         self.mock_cluster_update.side_effect = self._sim_rpc_cluster_update
         self.addCleanup(p.stop)
 
-    def _sim_rpc_cluster_update(self, cluster, rollback=False):
+    def _sim_rpc_cluster_update(self, cluster, node_count, health_status,
+                                health_status_reason, rollback=False):
+        cluster.status = 'UPDATE_IN_PROGRESS'
+        cluster.health_status = health_status
+        cluster.health_status_reason = health_status_reason
+        default_ng_worker = cluster.default_ng_worker
+        default_ng_worker.node_count = node_count
+        default_ng_worker.save()
         cluster.save()
         return cluster
 
@@ -307,6 +320,42 @@ class TestPatch(api_base.FunctionalTest):
         self.assertEqual(self.cluster_obj.cluster_template_id,
                          response['cluster_template_id'])
 
+    @mock.patch('oslo_utils.timeutils.utcnow')
+    def test_replace_health_status_ok(self, mock_utcnow):
+        new_health_status = 'HEALTHY'
+        new_health_status_reason = {'api': 'ok'}
+        health_status_reason_dumps = jsonutils.dumps(new_health_status_reason)
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
+        mock_utcnow.return_value = test_time
+        old_node_count = self.cluster_obj.default_ng_worker.node_count
+        db_utils.create_test_nodegroup(cluster_id=self.cluster_obj.uuid,
+                                       name='non_default_ng')
+        response = self.patch_json('/clusters/%s' % self.cluster_obj.uuid,
+                                   [{'path': '/health_status',
+                                     'value': new_health_status,
+                                     'op': 'replace'},
+                                    {'path': '/health_status_reason',
+                                     'value': health_status_reason_dumps,
+                                     'op': 'replace'}])
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_code)
+
+        response = self.get_json('/clusters/%s' % self.cluster_obj.uuid)
+        self.assertEqual(new_health_status, response['health_status'])
+        self.assertEqual(new_health_status_reason,
+                         response['health_status_reason'])
+
+        new_node_count = self.cluster_obj.default_ng_worker.node_count
+        self.assertEqual(old_node_count, new_node_count)
+
+        return_updated_at = timeutils.parse_isotime(
+            response['updated_at']).replace(tzinfo=None)
+        self.assertEqual(test_time, return_updated_at)
+        # Assert nothing else was changed
+        self.assertEqual(self.cluster_obj.uuid, response['uuid'])
+        self.assertEqual(self.cluster_obj.cluster_template_id,
+                         response['cluster_template_id'])
+
     @mock.patch('oslo_utils.timeutils.utcnow')
     def test_replace_ok_by_name(self, mock_utcnow):
         new_node_count = 4
@@ -437,23 +486,29 @@ class TestPatch(api_base.FunctionalTest):
         self.assertTrue(response.json['errors'])
 
     def test_update_cluster_with_rollback_enabled(self):
+        node_count = 4
         response = self.patch_json(
             '/clusters/%s/?rollback=True' % self.cluster_obj.uuid,
-            [{'path': '/node_count', 'value': 4,
+            [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
-        self.mock_cluster_update.assert_called_once_with(mock.ANY, True)
+        self.mock_cluster_update.assert_called_once_with(
+            mock.ANY, node_count, self.cluster_obj.health_status,
+            self.cluster_obj.health_status_reason, True)
         self.assertEqual(202, response.status_code)
 
     def test_update_cluster_with_rollback_disabled(self):
+        node_count = 4
         response = self.patch_json(
             '/clusters/%s/?rollback=False' % self.cluster_obj.uuid,
-            [{'path': '/node_count', 'value': 4,
+            [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
-        self.mock_cluster_update.assert_called_once_with(mock.ANY, False)
+        self.mock_cluster_update.assert_called_once_with(
+            mock.ANY, node_count, self.cluster_obj.health_status,
+            self.cluster_obj.health_status_reason, False)
         self.assertEqual(202, response.status_code)
 
     def test_remove_ok(self):
@@ -496,6 +551,19 @@ class TestPatch(api_base.FunctionalTest):
         self.assertEqual(400, response.status_code)
         self.assertTrue(response.json['errors'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_update_cluster_as_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster(self.context, uuid=temp_uuid)
+        self.context.is_admin = True
+        response = self.patch_json('/clusters/%s' % temp_uuid,
+                                   [{'path': '/node_count',
+                                     'value': 4,
+                                     'op': 'replace'}])
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_code)
+
 
 class TestPost(api_base.FunctionalTest):
     def setUp(self):
@@ -510,7 +578,8 @@ class TestPost(api_base.FunctionalTest):
         self.mock_valid_os_res = p.start()
         self.addCleanup(p.stop)
 
-    def _simulate_cluster_create(self, cluster, create_timeout):
+    def _simulate_cluster_create(self, cluster, master_count, node_count,
+                                 create_timeout):
         cluster.create()
         return cluster
 
@@ -549,7 +618,8 @@ class TestPost(api_base.FunctionalTest):
     def test_create_cluster_set_project_id_and_user_id(self):
         bdict = apiutils.cluster_post_data()
 
-        def _simulate_rpc_cluster_create(cluster, create_timeout):
+        def _simulate_rpc_cluster_create(cluster, master_count, node_count,
+                                         create_timeout):
             self.assertEqual(self.context.project_id, cluster.project_id)
             self.assertEqual(self.context.user_id, cluster.user_id)
             cluster.create()
@@ -681,7 +751,8 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual(202, response.status_int)
 
     def test_create_cluster_with_no_timeout(self):
-        def _simulate_rpc_cluster_create(cluster, create_timeout):
+        def _simulate_rpc_cluster_create(cluster, master_count, node_count,
+                                         create_timeout):
             self.assertEqual(60, create_timeout)
             cluster.create()
             return cluster
@@ -898,6 +969,53 @@ class TestPost(api_base.FunctionalTest):
         # Verify flavor_id from ClusterTemplate is used
         self.assertEqual('m1.small', cluster[0].flavor_id)
 
+    def test_create_cluster_with_cinder_csi_disabled(self):
+        self.cluster_template.volume_driver = 'cinder'
+        self.cluster_template.save()
+        cluster_labels = {'cinder_csi_enabled': 'false'}
+        bdict = apiutils.cluster_post_data(labels=cluster_labels)
+        note = 'in-tree Cinder volume driver is deprecated'
+        with self.assertWarnsRegex(DeprecationWarning, note):
+            response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+
+    def test_create_cluster_without_merge_labels(self):
+        self.cluster_template.labels = {'label1': 'value1', 'label2': 'value2'}
+        self.cluster_template.save()
+        cluster_labels = {'label2': 'value3', 'label4': 'value4'}
+        bdict = apiutils.cluster_post_data(labels=cluster_labels)
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual(cluster_labels, cluster[0].labels)
+
+    def test_create_cluster_with_merge_labels(self):
+        self.cluster_template.labels = {'label1': 'value1', 'label2': 'value2'}
+        self.cluster_template.save()
+        cluster_labels = {'label2': 'value3', 'label4': 'value4'}
+        bdict = apiutils.cluster_post_data(labels=cluster_labels,
+                                           merge_labels=True)
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        expected = self.cluster_template.labels
+        expected.update(cluster_labels)
+        self.assertEqual(expected, cluster[0].labels)
+
+    def test_create_cluster_with_merge_labels_no_labels(self):
+        self.cluster_template.labels = {'label1': 'value1', 'label2': 'value2'}
+        self.cluster_template.save()
+        bdict = apiutils.cluster_post_data(merge_labels=True)
+        del bdict['labels']
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual(self.cluster_template.labels, cluster[0].labels)
+
 
 class TestDelete(api_base.FunctionalTest):
     def setUp(self):
@@ -950,6 +1068,16 @@ class TestDelete(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['errors'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_delete_cluster_as_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster(self.context, uuid=temp_uuid)
+        self.context.is_admin = True
+        response = self.delete('/clusters/%s' % temp_uuid,
+                               expect_errors=True)
+        self.assertEqual(204, response.status_int)
+
 
 class TestClusterPolicyEnforcement(api_base.FunctionalTest):
     def setUp(self):
@@ -1000,6 +1128,9 @@ class TestClusterPolicyEnforcement(api_base.FunctionalTest):
     def _simulate_cluster_delete(self, cluster_uuid):
         cluster = objects.Cluster.get_by_uuid(self.context, cluster_uuid)
         cluster.destroy()
+        ngs = objects.NodeGroup.list(self.context, cluster_uuid)
+        for ng in ngs:
+            ng.destroy()
 
     def test_policy_disallow_delete(self):
         p = mock.patch.object(rpcapi.API, 'cluster_delete')
