@@ -11,8 +11,8 @@
 #    limitations under the License.
 
 import datetime
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
@@ -35,9 +35,23 @@ class TestClusterTemplateObject(base.TestCase):
     def test_cluster_template_init(self):
         cluster_template_dict = apiutils.cluster_template_post_data()
         del cluster_template_dict['image_id']
+        del cluster_template_dict['registry_enabled']
+        del cluster_template_dict['tls_disabled']
+        del cluster_template_dict['public']
+        del cluster_template_dict['server_type']
+        del cluster_template_dict['master_lb_enabled']
+        del cluster_template_dict['floating_ip_enabled']
+        del cluster_template_dict['hidden']
         cluster_template = api_cluster_template.ClusterTemplate(
             **cluster_template_dict)
         self.assertEqual(wtypes.Unset, cluster_template.image_id)
+        self.assertFalse(cluster_template.registry_enabled)
+        self.assertFalse(cluster_template.tls_disabled)
+        self.assertFalse(cluster_template.public)
+        self.assertEqual('vm', cluster_template.server_type)
+        self.assertFalse(cluster_template.master_lb_enabled)
+        self.assertTrue(cluster_template.floating_ip_enabled)
+        self.assertFalse(cluster_template.hidden)
 
 
 class TestListClusterTemplate(api_base.FunctionalTest):
@@ -50,7 +64,7 @@ class TestListClusterTemplate(api_base.FunctionalTest):
                                'image_id', 'registry_enabled', 'no_proxy',
                                'keypair_id', 'https_proxy', 'tls_disabled',
                                'public', 'labels', 'master_flavor_id',
-                               'volume_driver', 'insecure_registry')
+                               'volume_driver', 'insecure_registry', 'hidden')
 
     def test_empty(self):
         response = self.get_json('/clustertemplates')
@@ -86,6 +100,33 @@ class TestListClusterTemplate(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['errors'])
 
+    def test_get_one_by_uuid(self):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster_template(self.context, uuid=temp_uuid)
+        response = self.get_json(
+            '/clustertemplates/%s' % temp_uuid)
+        self.assertEqual(temp_uuid, response['uuid'])
+
+    def test_get_one_by_uuid_not_found(self):
+        temp_uuid = uuidutils.generate_uuid()
+        response = self.get_json(
+            '/clustertemplates/%s' % temp_uuid,
+            expect_errors=True)
+        self.assertEqual(404, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['errors'])
+
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_get_one_by_uuid_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster_template(self.context, uuid=temp_uuid,
+                                               project_id=temp_uuid)
+        self.context.is_admin = True
+        response = self.get_json(
+            '/clustertemplates/%s' % temp_uuid)
+        self.assertEqual(temp_uuid, response['uuid'])
+
     def test_get_one_by_name_multiple_cluster_template(self):
         obj_utils.create_test_cluster_template(
             self.context, name='test_clustertemplate',
@@ -114,6 +155,18 @@ class TestListClusterTemplate(api_base.FunctionalTest):
         self.assertEqual(bm_list[-1].uuid,
                          response['clustertemplates'][0]['uuid'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_get_all_with_all_projects(self, mock_context, mock_policy):
+        for id_ in range(4):
+            obj_utils.create_test_cluster_template(
+                self.context, id=id_, project_id=id_,
+                uuid=uuidutils.generate_uuid())
+
+        self.context.is_admin = True
+        response = self.get_json('/clustertemplates')
+        self.assertEqual(4, len(response['clustertemplates']))
+
     def test_detail(self):
         cluster_template = obj_utils.create_test_cluster_template(self.context)
         response = self.get_json('/clustertemplates/detail')
@@ -164,9 +217,9 @@ class TestListClusterTemplate(api_base.FunctionalTest):
         self.assertIn('links', response.keys())
         self.assertEqual(2, len(response['links']))
         self.assertIn(uuid, response['links'][0]['href'])
-        for l in response['links']:
-            bookmark = l['rel'] == 'bookmark'
-            self.assertTrue(self.validate_link(l['href'],
+        for link in response['links']:
+            bookmark = link['rel'] == 'bookmark'
+            self.assertTrue(self.validate_link(link['href'],
                                                bookmark=bookmark))
 
     def test_collection_links(self):
@@ -212,7 +265,8 @@ class TestPatch(api_base.FunctionalTest):
             public=False,
             docker_volume_size=20,
             coe='swarm',
-            labels={'key1': 'val1', 'key2': 'val2'}
+            labels={'key1': 'val1', 'key2': 'val2'},
+            hidden=False
         )
 
     def test_update_not_found(self):
@@ -233,8 +287,8 @@ class TestPatch(api_base.FunctionalTest):
 
         response = self.patch_json('/clustertemplates/%s' %
                                    cluster_template.uuid,
-                                   [{'path': '/name',
-                                     'value': 'cluster_model_example_B',
+                                   [{'path': '/network_driver',
+                                     'value': 'flannel',
                                      'op': 'replace'}],
                                    expect_errors=True)
         self.assertEqual(400, response.status_int)
@@ -243,6 +297,19 @@ class TestPatch(api_base.FunctionalTest):
         self.assertIn(cluster_template.uuid,
                       response.json['errors'][0]['detail'])
 
+    def test_update_cluster_template_name_with_cluster(self):
+        cluster_template = obj_utils.create_test_cluster_template(self.context)
+        obj_utils.create_test_cluster(
+            self.context, cluster_template_id=cluster_template.uuid)
+
+        response = self.patch_json('/clustertemplates/%s' %
+                                   cluster_template.uuid,
+                                   [{'path': '/name',
+                                     'value': 'cluster_model_example_B',
+                                     'op': 'replace'}],
+                                   expect_errors=True)
+        self.assertEqual(200, response.status_int)
+
     @mock.patch.object(magnum_policy, 'enforce')
     def test_update_public_cluster_template_success(self, mock_policy):
         mock_policy.return_value = True
@@ -265,7 +332,10 @@ class TestPatch(api_base.FunctionalTest):
                           [{'path': '/public', 'value': True,
                             'op': 'replace'}])
 
-    def test_update_cluster_template_with_cluster_allow_update(self):
+    @mock.patch.object(magnum_policy, 'enforce')
+    def test_update_cluster_template_with_cluster_allow_update(self,
+                                                               mock_policy):
+        mock_policy.return_value = True
         cluster_template = obj_utils.create_test_cluster_template(self.context)
         obj_utils.create_test_cluster(
             self.context, cluster_template_id=cluster_template.uuid)
@@ -280,14 +350,79 @@ class TestPatch(api_base.FunctionalTest):
                                  self.cluster_template.uuid)
         self.assertEqual(response['public'], True)
 
+    @mock.patch.object(magnum_policy, 'enforce')
+    def test_update_hidden_cluster_template_success(self, mock_policy):
+        mock_policy.return_value = True
+        response = self.patch_json('/clustertemplates/%s' %
+                                   self.cluster_template.uuid,
+                                   [{'path': '/hidden', 'value': True,
+                                     'op': 'replace'}])
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(200, response.status_code)
+
+        response = self.get_json('/clustertemplates/%s' %
+                                 self.cluster_template.uuid)
+        self.assertTrue(response['hidden'])
+
+    @mock.patch.object(magnum_policy, 'enforce')
+    def test_update_hidden_cluster_template_fail(self, mock_policy):
+        mock_policy.return_value = False
+        self.assertRaises(AppError, self.patch_json,
+                          '/clustertemplates/%s' % self.cluster_template.uuid,
+                          [{'path': '/hidden', 'value': True,
+                            'op': 'replace'}])
+
+    @mock.patch.object(magnum_policy, 'enforce')
+    def test_update_cluster_template_hidden_with_cluster_allow_update(
+            self, mock_policy):
+        mock_policy.return_value = True
+        cluster_template = obj_utils.create_test_cluster_template(self.context)
+        obj_utils.create_test_cluster(
+            self.context, cluster_template_id=cluster_template.uuid)
+        response = self.patch_json('/clustertemplates/%s' %
+                                   cluster_template.uuid,
+                                   [{'path': '/hidden',
+                                     'value': True,
+                                     'op': 'replace'}],
+                                   expect_errors=True)
+        self.assertEqual(200, response.status_int)
+        response = self.get_json('/clustertemplates/%s' %
+                                 self.cluster_template.uuid)
+        self.assertEqual(response['hidden'], True)
+
+    def test_update_cluster_template_with_devicemapper(self):
+        cluster_template = obj_utils.create_test_cluster_template(self.context)
+        note = 'deprecated in favor of overlay2'
+        with self.assertWarnsRegex(DeprecationWarning, note):
+            response = self.patch_json('/clustertemplates/%s' %
+                                       cluster_template.uuid,
+                                       [{'path': '/docker_storage_driver',
+                                         'value': 'devicemapper',
+                                         'op': 'replace'}],
+                                       expect_errors=True)
+        self.assertEqual(200, response.status_int)
+
+    def test_update_cluster_template_replace_labels_success(self):
+        cluster_template = obj_utils.create_test_cluster_template(self.context)
+        response = self.patch_json('/clustertemplates/%s' %
+                                   cluster_template.uuid,
+                                   [{'path': '/labels',
+                                     'value': '{\'etcd_volume_size\': \'1\'}',
+                                     'op': 'replace'}],
+                                   expect_errors=True)
+        self.assertEqual(200, response.status_int)
+        response = self.get_json('/clustertemplates/%s' %
+                                 self.cluster_template.uuid)
+        self.assertEqual(response['labels'], {'etcd_volume_size': '1'})
+
     def test_update_cluster_template_with_cluster_not_allow_update(self):
         cluster_template = obj_utils.create_test_cluster_template(self.context)
         obj_utils.create_test_cluster(
             self.context, cluster_template_id=cluster_template.uuid)
         response = self.patch_json('/clustertemplates/%s' %
                                    cluster_template.uuid,
-                                   [{'path': '/name',
-                                     'value': 'new_name',
+                                   [{'path': '/network_driver',
+                                     'value': 'calico',
                                      'op': 'replace'}],
                                    expect_errors=True)
         self.assertEqual(400, response.status_code)
@@ -442,7 +577,7 @@ class TestPatch(api_base.FunctionalTest):
         self.assertTrue(response.json['errors'])
 
     def test_remove_mandatory_property_fail(self):
-        mandatory_properties = ('/image_id', '/keypair_id', '/coe',
+        mandatory_properties = ('/image_id', '/coe',
                                 '/external_network_id', '/server_type',
                                 '/tls_disabled', '/public',
                                 '/registry_enabled',
@@ -474,6 +609,20 @@ class TestPatch(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['errors'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_update_cluster_template_as_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster_template(self.context, uuid=temp_uuid,
+                                               project_id=temp_uuid)
+        self.context.is_admin = True
+        response = self.patch_json('/clustertemplates/%s' % temp_uuid,
+                                   [{'path': '/name',
+                                     'value': 'cluster_model_example_B',
+                                     'op': 'replace'}],
+                                   expect_errors=True)
+        self.assertEqual(200, response.status_int)
+
 
 class TestPost(api_base.FunctionalTest):
 
@@ -576,7 +725,6 @@ class TestPost(api_base.FunctionalTest):
         self._create_model_raises_app_error(coe='osomatsu')
 
     def test_create_cluster_template_with_invalid_docker_volume_size(self):
-        self._create_model_raises_app_error(docker_volume_size=0)
         self._create_model_raises_app_error(docker_volume_size=-1)
         self._create_model_raises_app_error(
             docker_volume_size=1,
@@ -637,7 +785,9 @@ class TestPost(api_base.FunctionalTest):
                                             'os_distro': 'fedora-atomic'}
             bdict = apiutils.cluster_template_post_data(
                 docker_volume_size=1, docker_storage_driver="overlay")
-            response = self.post_json('/clustertemplates', bdict)
+            note = 'deprecated in favor of overlay2'
+            with self.assertWarnsRegex(DeprecationWarning, note):
+                response = self.post_json('/clustertemplates', bdict)
             self.assertEqual(bdict['docker_volume_size'],
                              response.json['docker_volume_size'])
             cc_mock.assert_called_once_with(mock.ANY)
@@ -653,7 +803,7 @@ class TestPost(api_base.FunctionalTest):
         mock_image_data.return_value = {'name': 'mock_name',
                                         'os_distro': 'fedora-atomic'}
         for k, v in cluster_template_config_dict.items():
-                    cfg.CONF.set_override(k, v, 'cluster_template')
+            cfg.CONF.set_override(k, v, 'cluster_template')
         with mock.patch.object(
                 self.dbapi, 'create_cluster_template',
                 wraps=self.dbapi.create_cluster_template) as cc_mock:
@@ -800,6 +950,40 @@ class TestPost(api_base.FunctionalTest):
             self.assertNotIn('id', cc_mock.call_args[0][0])
             self.assertFalse(cc_mock.call_args[0][0]['public'])
 
+    @mock.patch('magnum.api.attr_validator.validate_image')
+    @mock.patch.object(magnum_policy, 'enforce')
+    def test_create_cluster_template_hidden_success(self, mock_policy,
+                                                    mock_image_data):
+        with mock.patch.object(
+                self.dbapi, 'create_cluster_template',
+                wraps=self.dbapi.create_cluster_template) as cc_mock:
+            mock_policy.return_value = True
+            mock_image_data.return_value = {'name': 'mock_name',
+                                            'os_distro': 'fedora-atomic'}
+            bdict = apiutils.cluster_template_post_data(hidden=True)
+            response = self.post_json('/clustertemplates', bdict)
+            self.assertTrue(response.json['hidden'])
+            mock_policy.assert_called_with(mock.ANY,
+                                           "clustertemplate:publish",
+                                           None, do_raise=False)
+            cc_mock.assert_called_once_with(mock.ANY)
+            self.assertNotIn('id', cc_mock.call_args[0][0])
+            self.assertTrue(cc_mock.call_args[0][0]['hidden'])
+
+    @mock.patch('magnum.api.attr_validator.validate_image')
+    @mock.patch.object(magnum_policy, 'enforce')
+    def test_create_cluster_template_hidden_fail(self, mock_policy,
+                                                 mock_image_data):
+        with mock.patch.object(self.dbapi, 'create_cluster_template',
+                               wraps=self.dbapi.create_cluster_template):
+            # make policy enforcement fail
+            mock_policy.return_value = False
+            mock_image_data.return_value = {'name': 'mock_name',
+                                            'os_distro': 'fedora-atomic'}
+            bdict = apiutils.cluster_template_post_data(hidden=True)
+            self.assertRaises(AppError, self.post_json, '/clustertemplates',
+                              bdict)
+
     @mock.patch('magnum.api.attr_validator.validate_image')
     def test_create_cluster_template_with_no_os_distro_image(self,
                                                              mock_image_data):
@@ -860,12 +1044,15 @@ class TestPost(api_base.FunctionalTest):
                                   expect_errors=True)
         self.assertEqual(400, response.status_int)
 
-    def test_create_cluster_template_without_keypair_id(self):
+    @mock.patch('magnum.api.attr_validator.validate_image')
+    def test_create_cluster_template_without_keypair_id(self,
+                                                        mock_image_data):
+        mock_image_data.return_value = {'name': 'mock_name',
+                                        'os_distro': 'fedora-atomic'}
         bdict = apiutils.cluster_template_post_data()
         del bdict['keypair_id']
-        response = self.post_json('/clustertemplates', bdict,
-                                  expect_errors=True)
-        self.assertEqual(400, response.status_int)
+        response = self.post_json('/clustertemplates', bdict)
+        self.assertEqual(201, response.status_int)
 
     @mock.patch('magnum.api.attr_validator.validate_image')
     def test_create_cluster_template_with_dns(self,
@@ -948,6 +1135,38 @@ class TestPost(api_base.FunctionalTest):
             self.assertEqual(201, resp.status_int)
             self.assertIsNotNone(resp.json['name'])
 
+    def test_create_cluster_with_disabled_driver(self):
+        cfg.CONF.set_override('disabled_drivers',
+                              ['mesos_ubuntu_v1'],
+                              group='drivers')
+        bdict = apiutils.cluster_template_post_data(coe="mesos")
+        self.assertRaises(AppError, self.post_json, '/clustertemplates',
+                          bdict)
+
+    @mock.patch('magnum.api.attr_validator.validate_image')
+    @mock.patch('oslo_utils.timeutils.utcnow')
+    def test_create_cluster_template_with_multi_dns(self, mock_utcnow,
+                                                    mock_image_data):
+        bdict = apiutils.cluster_template_post_data(
+            dns_nameserver="8.8.8.8,114.114.114.114")
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
+        mock_utcnow.return_value = test_time
+        mock_image_data.return_value = {'name': 'mock_name',
+                                        'os_distro': 'fedora-atomic'}
+
+        response = self.post_json('/clustertemplates', bdict)
+        self.assertEqual(201, response.status_int)
+        # Check location header
+        self.assertIsNotNone(response.location)
+        expected_location = '/v1/clustertemplates/%s' % bdict['uuid']
+        self.assertEqual(expected_location,
+                         urlparse.urlparse(response.location).path)
+        self.assertEqual(bdict['uuid'], response.json['uuid'])
+        self.assertNotIn('updated_at', response.json.keys)
+        return_created_at = timeutils.parse_isotime(
+            response.json['created_at']).replace(tzinfo=None)
+        self.assertEqual(test_time, return_created_at)
+
 
 class TestDelete(api_base.FunctionalTest):
 
@@ -1008,6 +1227,17 @@ class TestDelete(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['errors'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_delete_cluster_template_as_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster_template(self.context, uuid=temp_uuid,
+                                               project_id=temp_uuid)
+        self.context.is_admin = True
+        response = self.delete('/clustertemplates/%s' % temp_uuid,
+                               expect_errors=True)
+        self.assertEqual(204, response.status_int)
+
 
 class TestClusterTemplatePolicyEnforcement(api_base.FunctionalTest):
 
