@@ -14,8 +14,8 @@
 
 
 from glanceclient import exc as glance_exception
-import mock
 from novaclient import exceptions as nova_exc
+from unittest import mock
 
 from magnum.api import attr_validator
 from magnum.common import exception
@@ -46,7 +46,7 @@ class TestAttrValidator(base.BaseTestCase):
         mock_os_cli = mock.MagicMock()
         mock_os_cli.nova.return_value = mock_nova
         attr_validator.validate_flavor(mock_os_cli, None)
-        self.assertFalse(mock_nova.flavors.list.called)
+        self.assertEqual(False, mock_nova.flavors.list.called)
 
     def test_validate_flavor_with_invalid_flavor(self):
         mock_flavor = mock.MagicMock()
@@ -94,6 +94,15 @@ class TestAttrValidator(base.BaseTestCase):
                           attr_validator.validate_external_network,
                           mock_os_cli, 'test_ext_net')
 
+    def test_validate_keypair_with_no_keypair(self):
+        mock_keypair = mock.MagicMock()
+        mock_keypair.id = None
+        mock_nova = mock.MagicMock()
+        mock_nova.keypairs.get.return_value = mock_keypair
+        mock_os_cli = mock.MagicMock()
+        mock_os_cli.nova.return_value = mock_nova
+        attr_validator.validate_keypair(mock_os_cli, None)
+
     def test_validate_keypair_with_valid_keypair(self):
         mock_keypair = mock.MagicMock()
         mock_keypair.id = 'test-keypair'
@@ -186,6 +195,20 @@ class TestAttrValidator(base.BaseTestCase):
                           attr_validator.validate_labels_isolation,
                           fake_labels)
 
+    def test_validate_labels_strategy_valid(self):
+        fake_labels = {'swarm_strategy': 'spread'}
+        attr_validator.validate_labels_strategy(fake_labels)
+
+    def test_validate_labels_strategy_missing(self):
+        fake_labels = {'strategy': 'spread'}
+        attr_validator.validate_labels_strategy(fake_labels)
+
+    def test_validate_labels_strategy_invalid(self):
+        fake_labels = {'swarm_strategy': 'invalid'}
+        self.assertRaises(exception.InvalidParameterValue,
+                          attr_validator.validate_labels_strategy,
+                          fake_labels)
+
     @mock.patch('magnum.api.utils.get_openstack_resource')
     def test_validate_image_with_valid_image_by_name(self, mock_os_res):
         mock_image = {'name': 'fedora-21-atomic-5',
@@ -297,3 +320,25 @@ class TestAttrValidator(base.BaseTestCase):
         mock_context = mock.MagicMock()
         attr_validator.validate_os_resources(mock_context,
                                              mock_cluster_template)
+
+    @mock.patch('magnum.common.clients.OpenStackClients')
+    def test_validate_os_resources_with_cluster(self, mock_os_cli):
+        mock_cluster_template = {}
+        mock_cluster = {
+            'keypair': 'test-keypair', 'labels': {'lab1': 'val1'},
+            'image_id': 'e33f0988-1730-405e-8401-30cbc8535302'
+        }
+        mock_keypair = mock.MagicMock()
+        mock_keypair.id = 'test-keypair'
+        mock_image = {'name': 'fedora-21-atomic-5',
+                      'id': 'e33f0988-1730-405e-8401-30cbc8535302',
+                      'os_distro': 'fedora-atomic'}
+        mock_nova = mock.MagicMock()
+        mock_nova.keypairs.get.return_value = mock_keypair
+        mock_nova.images.get.return_value = mock_image
+        mock_os_cli = mock.MagicMock()
+        mock_os_cli.nova.return_value = mock_nova
+        mock_context = mock.MagicMock()
+        attr_validator.validate_os_resources(mock_context,
+                                             mock_cluster_template,
+                                             mock_cluster)
