@@ -13,161 +13,72 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
-from oslo_config import cfg
+from unittest import mock
+
 from six.moves import reload_module
 
 from magnum.api import validation as v
 from magnum.common import exception
+import magnum.conf
 from magnum import objects
 from magnum.tests import base
 from magnum.tests.unit.objects import utils as obj_utils
 
+CONF = magnum.conf.CONF
+
 
 class TestValidation(base.BaseTestCase):
 
-    def _test_enforce_cluster_types(
-            self,
-            mock_cluster_get_by_uuid,
-            mock_pecan_request,
-            cluster_type,
-            allowed_cluster_types,
-            assert_raised=False,
-            *args):
+    def _test_enforce_cluster_type_supported(
+            self, mock_cluster_template_get_by_uuid, mock_cluster_get_by_uuid,
+            mock_pecan_request, cluster_type, assert_raised=False):
 
-        @v.enforce_cluster_types(*allowed_cluster_types)
-        def test(self, *args):
-            if hasattr(args[0], 'cluster_uuid'):
-                return args[0].name
-            else:
-                return args[1]
+        @v.enforce_cluster_type_supported()
+        def test(self, cluster):
+            pass
+
+        server_type, cluster_distro, coe = cluster_type
+        cluster_template = obj_utils.get_test_cluster_template(
+            mock_pecan_request.context, uuid='cluster_template_id',
+            coe=coe, cluster_distro=cluster_distro, server_type=server_type)
+        mock_cluster_template_get_by_uuid.return_value = cluster_template
 
-        context = mock_pecan_request.context
         cluster = mock.MagicMock()
         cluster.cluster_template_id = 'cluster_template_id'
-        cluster_template = obj_utils.get_test_cluster_template(
-            context, uuid='cluster_template_id', coe=cluster_type)
         cluster.cluster_template = cluster_template
-
         mock_cluster_get_by_uuid.return_value = cluster
 
         if assert_raised:
-            self.assertRaises(
-                exception.InvalidParameterValue, test, self, *args)
+            return self.assertRaises(
+                exception.ClusterTypeNotSupported, test, self, cluster)
         else:
-            ret = test(self, *args)
-            if hasattr(args[0], 'cluster_uuid'):
-                mock_cluster_get_by_uuid.assert_called_once_with(
-                    context, args[0].cluster_uuid)
-                self.assertEqual(args[0].name, ret)
-            else:
-                mock_cluster_get_by_uuid.assert_called_once_with(
-                    context, args[1])
-                self.assertEqual(args[1], ret)
+            self.assertIsNone(test(self, cluster))
 
     @mock.patch('pecan.request')
     @mock.patch('magnum.objects.Cluster.get_by_uuid')
-    def test_enforce_cluster_types_one_allowed(
-            self,
-            mock_cluster_get_by_uuid,
+    @mock.patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    def test_enforce_cluster_type_supported(
+            self, mock_cluster_template_get_by_uuid, mock_cluster_get_by_uuid,
             mock_pecan_request):
 
-        obj = mock.MagicMock()
-        obj.name = 'test_object'
-        obj.cluster_uuid = 'cluster_uuid'
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['swarm']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types, False, obj)
+        cluster_type = ('vm', 'fedora-atomic', 'kubernetes')
+        self._test_enforce_cluster_type_supported(
+            mock_cluster_template_get_by_uuid, mock_cluster_get_by_uuid,
+            mock_pecan_request, cluster_type)
 
     @mock.patch('pecan.request')
     @mock.patch('magnum.objects.Cluster.get_by_uuid')
-    def test_enforce_cluster_types_two_allowed(
-            self,
-            mock_cluster_get_by_uuid,
+    @mock.patch('magnum.objects.ClusterTemplate.get_by_uuid')
+    def test_enforce_cluster_type_not_supported(
+            self, mock_cluster_template_get_by_uuid, mock_cluster_get_by_uuid,
             mock_pecan_request):
 
-        obj = mock.MagicMock()
-        obj.name = 'test_object'
-        obj.cluster_uuid = 'cluster_uuid'
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['swarm', 'mesos']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types, False, obj)
-
-    @mock.patch('pecan.request')
-    @mock.patch('magnum.objects.Cluster.get_by_uuid')
-    def test_enforce_cluster_types_not_allowed(
-            self,
-            mock_cluster_get_by_uuid,
-            mock_pecan_request):
-
-        obj = mock.MagicMock()
-        obj.name = 'test_object'
-        obj.cluster_uuid = 'cluster_uuid'
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['mesos']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types,
-            True, obj)
-
-    @mock.patch('pecan.request')
-    @mock.patch('magnum.objects.Cluster.get_by_uuid')
-    def test_enforce_cluster_types_with_cluster_uuid(self,
-                                                     mock_cluster_get_by_uuid,
-                                                     mock_pecan_request):
-
-        cluster_ident = 'e74c40e0-d825-11e2-a28f-0800200c9a66'
-
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['swarm']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types, False,
-            None, cluster_ident)
-
-    @mock.patch('pecan.request')
-    @mock.patch('magnum.objects.Cluster.get_by_uuid')
-    def test_enforce_cluster_types_with_cluster_uuid_not_allowed(
-            self, mock_cluster_get_by_uuid, mock_pecan_request):
-
-        cluster_ident = 'e74c40e0-d825-11e2-a28f-0800200c9a66'
-
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['mesos']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types, True,
-            None, cluster_ident)
-
-    @mock.patch('pecan.request')
-    @mock.patch('magnum.objects.Cluster.get_by_name')
-    def test_enforce_cluster_types_with_cluster_name(
-            self, mock_cluster_get_by_uuid, mock_pecan_request):
-
-        cluster_ident = 'cluster_name'
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['swarm']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types, False,
-            None, cluster_ident)
-
-    @mock.patch('pecan.request')
-    @mock.patch('magnum.objects.Cluster.get_by_name')
-    def test_enforce_cluster_types_with_cluster_name_not_allowed(
-            self, mock_cluster_get_by_uuid, mock_pecan_request):
-
-        cluster_ident = 'cluster_name'
-        cluster_type = 'swarm'
-        allowed_cluster_types = ['mesos']
-        self._test_enforce_cluster_types(
-            mock_cluster_get_by_uuid, mock_pecan_request,
-            cluster_type, allowed_cluster_types, True,
-            None, cluster_ident)
+        cluster_type = ('vm', 'foo', 'kubernetes')
+        exc = self._test_enforce_cluster_type_supported(
+            mock_cluster_template_get_by_uuid, mock_cluster_get_by_uuid,
+            mock_pecan_request, cluster_type, assert_raised=True)
+        self.assertEqual('Cluster type (vm, foo, kubernetes) not supported.',
+                         exc.message)
 
     def _test_enforce_network_driver_types_create(
             self,
@@ -181,7 +92,7 @@ class TestValidation(base.BaseTestCase):
             pass
 
         for key, val in network_driver_config_dict.items():
-            cfg.CONF.set_override(key, val, 'cluster_template')
+            CONF.set_override(key, val, 'cluster_template')
 
         cluster_template = mock.MagicMock()
         cluster_template.name = 'test_cluster_template'
@@ -269,7 +180,7 @@ class TestValidation(base.BaseTestCase):
             pass
 
         for key, val in network_driver_config_dict.items():
-            cfg.CONF.set_override(key, val, 'cluster_template')
+            CONF.set_override(key, val, 'cluster_template')
 
         cluster_template_ident = 'test_uuid_or_name'
 
@@ -350,6 +261,39 @@ class TestValidation(base.BaseTestCase):
             volume_driver_type='type',
             assert_raised=True)
 
+    def _test_enforce_server_type(
+            self,
+            server_type,
+            coe='kubernetes',
+            assert_raised=False):
+
+        @v.enforce_server_type()
+        def test(self, cluster_template):
+            pass
+
+        cluster_template = obj_utils.get_test_cluster_template(
+            {}, name='test_cluster_template', coe=coe,
+            server_type=server_type)
+
+        if assert_raised:
+            self.assertRaises(exception.InvalidParameterValue,
+                              test, self, cluster_template)
+        else:
+            test(self, cluster_template)
+
+    def test_enforce_server_type_valid_vm(self):
+        self._test_enforce_server_type(
+            server_type='vm')
+
+    def test_enforce_server_type_valid_bm(self):
+        self._test_enforce_server_type(
+            server_type='bm')
+
+    def test_enforce_server_type_invalid(self):
+        self._test_enforce_server_type(
+            server_type='invalid',
+            assert_raised=True)
+
     @mock.patch('pecan.request')
     @mock.patch('magnum.api.utils.get_resource')
     def _test_enforce_volume_driver_types_update(
