@@ -10,32 +10,28 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from barbicanclient import client as barbicanclient
+from barbicanclient.v1 import client as barbicanclient
 from glanceclient import client as glanceclient
 from heatclient import client as heatclient
-import mock
 from neutronclient.v2_0 import client as neutronclient
 from novaclient import client as novaclient
-from oslo_config import cfg
+from unittest import mock
 
 from magnum.common import clients
 from magnum.common import exception
+import magnum.conf
 from magnum.tests import base
 
+CONF = magnum.conf.CONF
+
 
 class ClientsTest(base.BaseTestCase):
 
     def setUp(self):
         super(ClientsTest, self).setUp()
 
-        cfg.CONF.set_override('auth_uri', 'http://server.test:5000/v2.0',
-                              group='keystone_authtoken')
-        cfg.CONF.import_opt('api_version', 'magnum.common.clients',
-                            group='nova_client')
-        cfg.CONF.import_opt('api_version', 'magnum.common.clients',
-                            group='heat_client')
-        cfg.CONF.import_opt('api_version', 'magnum.common.clients',
-                            group='glance_client')
+        CONF.set_override('auth_uri', 'http://server.test:5000/v2.0',
+                          group='keystone_authtoken')
 
     @mock.patch.object(clients.OpenStackClients, 'keystone')
     def test_url_for(self, mock_keystone):
@@ -50,10 +46,10 @@ class ClientsTest(base.BaseTestCase):
     def test_magnum_url(self, mock_keystone):
         fake_region = 'fake_region'
         fake_endpoint = 'fake_endpoint'
-        cfg.CONF.set_override('region_name', fake_region,
-                              group='magnum_client')
-        cfg.CONF.set_override('endpoint_type', fake_endpoint,
-                              group='magnum_client')
+        CONF.set_override('region_name', fake_region,
+                          group='magnum_client')
+        CONF.set_override('endpoint_type', fake_endpoint,
+                          group='magnum_client')
         obj = clients.OpenStackClients(None)
         obj.magnum_url()
 
@@ -76,7 +72,7 @@ class ClientsTest(base.BaseTestCase):
         obj._heat = None
         obj.heat()
         mock_call.assert_called_once_with(
-            cfg.CONF.heat_client.api_version,
+            CONF.heat_client.api_version,
             endpoint='url_from_keystone', username=None,
             cert_file=None, token='3bcc3d3a03f44e3d8377f9247b0ad155',
             auth_url='keystone_url', ca_file=None, key_file=None,
@@ -89,13 +85,14 @@ class ClientsTest(base.BaseTestCase):
         self._test_clients_heat(None)
 
     def test_clients_heat_region(self):
-        cfg.CONF.set_override('region_name', 'myregion', group='heat_client')
+        CONF.set_override('region_name', 'myregion', group='heat_client')
         self._test_clients_heat('myregion')
 
     def test_clients_heat_noauth(self):
         con = mock.MagicMock()
         con.auth_token = None
         con.auth_token_info = None
+        con.trust_id = None
         auth_url = mock.PropertyMock(name="auth_url",
                                      return_value="keystone_url")
         type(con).auth_url = auth_url
@@ -133,7 +130,7 @@ class ClientsTest(base.BaseTestCase):
         obj._glance = None
         obj.glance()
         mock_call.assert_called_once_with(
-            cfg.CONF.glance_client.api_version,
+            CONF.glance_client.api_version,
             endpoint='url_from_keystone', username=None,
             token='3bcc3d3a03f44e3d8377f9247b0ad155',
             auth_url='keystone_url',
@@ -146,13 +143,14 @@ class ClientsTest(base.BaseTestCase):
         self._test_clients_glance(None)
 
     def test_clients_glance_region(self):
-        cfg.CONF.set_override('region_name', 'myregion', group='glance_client')
+        CONF.set_override('region_name', 'myregion', group='glance_client')
         self._test_clients_glance('myregion')
 
     def test_clients_glance_noauth(self):
         con = mock.MagicMock()
         con.auth_token = None
         con.auth_token_info = None
+        con.trust_id = None
         auth_url = mock.PropertyMock(name="auth_url",
                                      return_value="keystone_url")
         type(con).auth_url = auth_url
@@ -203,14 +201,15 @@ class ClientsTest(base.BaseTestCase):
         self._test_clients_barbican(None)
 
     def test_clients_barbican_region(self):
-        cfg.CONF.set_override('region_name', 'myregion',
-                              group='barbican_client')
+        CONF.set_override('region_name', 'myregion',
+                          group='barbican_client')
         self._test_clients_barbican('myregion')
 
     def test_clients_barbican_noauth(self):
         con = mock.MagicMock()
         con.auth_token = None
         con.auth_token_info = None
+        con.trust_id = None
         auth_url = mock.PropertyMock(name="auth_url",
                                      return_value="keystone_url")
         type(con).auth_url = auth_url
@@ -236,21 +235,27 @@ class ClientsTest(base.BaseTestCase):
         self.assertEqual(barbican, barbican_cached)
 
     @mock.patch.object(novaclient, 'Client')
+    @mock.patch.object(clients.OpenStackClients, 'keystone')
     @mock.patch.object(clients.OpenStackClients, 'url_for')
     @mock.patch.object(clients.OpenStackClients, 'auth_url')
     def _test_clients_nova(self, expected_region_name, mock_auth, mock_url,
-                           mock_call):
+                           mock_keystone, mock_call):
         mock_auth.__get__ = mock.Mock(return_value="keystone_url")
         con = mock.MagicMock()
-        con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
+        keystone = mock.MagicMock()
+        keystone.session = mock.MagicMock()
+        mock_keystone.return_value = keystone
         con.auth_url = "keystone_url"
         mock_url.return_value = "url_from_keystone"
         obj = clients.OpenStackClients(con)
         obj._nova = None
         obj.nova()
-        mock_call.assert_called_once_with(cfg.CONF.nova_client.api_version,
-                                          auth_token=con.auth_token,
-                                          cacert=None, insecure=False)
+        expected_kwargs = {'session': keystone.session,
+                           'endpoint_override': mock_url.return_value,
+                           'cacert': None,
+                           'insecure': False}
+        mock_call.assert_called_once_with(CONF.nova_client.api_version,
+                                          **expected_kwargs)
         mock_url.assert_called_once_with(service_type='compute',
                                          interface='publicURL',
                                          region_name=expected_region_name)
@@ -259,13 +264,14 @@ class ClientsTest(base.BaseTestCase):
         self._test_clients_nova(None)
 
     def test_clients_nova_region(self):
-        cfg.CONF.set_override('region_name', 'myregion', group='nova_client')
+        CONF.set_override('region_name', 'myregion', group='nova_client')
         self._test_clients_nova('myregion')
 
     def test_clients_nova_noauth(self):
         con = mock.MagicMock()
         con.auth_token = None
         con.auth_token_info = None
+        con.trust_id = None
         auth_url = mock.PropertyMock(name="auth_url",
                                      return_value="keystone_url")
         type(con).auth_url = auth_url
@@ -281,6 +287,7 @@ class ClientsTest(base.BaseTestCase):
         mock_auth.__get__ = mock.Mock(return_value="keystone_url")
         con = mock.MagicMock()
         con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
+        con.auth_token_info = "auth-token-info"
         con.auth_url = "keystone_url"
         mock_url.return_value = "url_from_keystone"
         obj = clients.OpenStackClients(con)
@@ -295,8 +302,8 @@ class ClientsTest(base.BaseTestCase):
     def _test_clients_neutron(self, expected_region_name, mock_auth, mock_url,
                               mock_call):
         fake_endpoint_type = 'fake_endpoint_type'
-        cfg.CONF.set_override('endpoint_type', fake_endpoint_type,
-                              group='neutron_client')
+        CONF.set_override('endpoint_type', fake_endpoint_type,
+                          group='neutron_client')
         mock_auth.__get__ = mock.Mock(return_value="keystone_url")
         con = mock.MagicMock()
         con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
@@ -319,14 +326,15 @@ class ClientsTest(base.BaseTestCase):
         self._test_clients_neutron(None)
 
     def test_clients_neutron_region(self):
-        cfg.CONF.set_override('region_name', 'myregion',
-                              group='neutron_client')
+        CONF.set_override('region_name', 'myregion',
+                          group='neutron_client')
         self._test_clients_neutron('myregion')
 
     def test_clients_neutron_noauth(self):
         con = mock.MagicMock()
         con.auth_token = None
         con.auth_token_info = None
+        con.trust_id = None
         auth_url = mock.PropertyMock(name="auth_url",
                                      return_value="keystone_url")
         type(con).auth_url = auth_url
