@@ -12,20 +12,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from docker import client as docker_py_client
-import mock
-from oslo_config import cfg
+import docker
+from unittest import mock
 
 from magnum.common import docker_utils
+import magnum.conf
 from magnum.tests import base
 
 
-CONF = cfg.CONF
-
-CONF.import_opt('docker_remote_api_version', 'magnum.common.docker_utils',
-                group='docker')
-CONF.import_opt('default_timeout', 'magnum.common.docker_utils',
-                group='docker')
+CONF = magnum.conf.CONF
 
 
 class TestDockerUtils(base.BaseTestCase):
@@ -35,9 +30,9 @@ class TestDockerUtils(base.BaseTestCase):
         def fake_version():
             return {'ApiVersion': '1.18'}
 
-        docker = mock.MagicMock()
-        docker.version.side_effect = fake_version
-        res = docker_utils.is_docker_api_version_atleast(docker, '1.19')
+        docker_client = mock.MagicMock()
+        docker_client.version.side_effect = fake_version
+        res = docker_utils.is_docker_api_version_atleast(docker_client, '1.21')
         self.assertFalse(res)
 
 
@@ -70,7 +65,7 @@ class DockerClientTestCase(base.BaseTestCase):
                          client.timeout)
 
     def test_docker_client_init_version(self):
-        expected_version = '1.16'
+        expected_version = '1.21'
         client = docker_utils.DockerHTTPClient(ver=expected_version)
 
         self.assertEqual(expected_version,
@@ -78,8 +73,8 @@ class DockerClientTestCase(base.BaseTestCase):
         self.assertEqual(CONF.docker.default_timeout,
                          client.timeout)
 
-    @mock.patch.object(docker_py_client.Client, 'inspect_container')
-    @mock.patch.object(docker_py_client.Client, 'containers')
+    @mock.patch.object(docker.APIClient, 'inspect_container')
+    @mock.patch.object(docker.APIClient, 'containers')
     def test_list_instances(self, mock_containers, mock_inspect):
         client = docker_utils.DockerHTTPClient()
 
@@ -95,8 +90,8 @@ class DockerClientTestCase(base.BaseTestCase):
         mock_containers.assert_called_once_with(all=True)
         mock_inspect.assert_has_calls([mock.call(x) for x in range(0, 3)])
 
-    @mock.patch.object(docker_py_client.Client, 'inspect_container')
-    @mock.patch.object(docker_py_client.Client, 'containers')
+    @mock.patch.object(docker.APIClient, 'inspect_container')
+    @mock.patch.object(docker.APIClient, 'containers')
     def test_list_instances_inspect(self, mock_containers, mock_inspect):
         client = docker_utils.DockerHTTPClient()
 
