@@ -18,7 +18,8 @@ from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives import serialization
 from cryptography import x509 as c_x509
 from cryptography.x509.oid import NameOID
-import mock
+from unittest import mock
+
 import six
 
 from magnum.common import exception
@@ -32,6 +33,7 @@ class TestX509(base.BaseTestCase):
         super(TestX509, self).setUp()
         self.issuer_name = six.u("fake-issuer")
         self.subject_name = six.u("fake-subject")
+        self.organization_name = six.u("fake-organization")
         self.ca_encryption_password = six.b("fake-ca-password")
         self.encryption_password = six.b("fake-password")
 
@@ -59,6 +61,7 @@ class TestX509(base.BaseTestCase):
         keypairs = operations.generate_client_certificate(
             self.issuer_name,
             self.subject_name,
+            self.organization_name,
             ca['private_key'],
             encryption_password=self.encryption_password,
             ca_key_password=self.ca_encryption_password,
@@ -221,6 +224,22 @@ class TestX509(base.BaseTestCase):
                         skip_validation=True)
         mock_six.assert_called_once_with(csr)
 
+    @mock.patch('cryptography.x509.load_pem_x509_csr')
+    def test_sign_empty_chars(self, mock_load_pem):
+        ca_key = self._generate_private_key()
+        private_key = self._generate_private_key()
+        csr_obj = self._build_csr(private_key)
+        csr = csr_obj.public_bytes(serialization.Encoding.PEM)
+        csr = six.text_type(csr.decode('utf-8'))
+
+        mock_load_pem.return_value = csr_obj
+        certificate = operations.sign(csr, self.issuer_name,
+                                      ca_key, skip_validation=True)
+
+        # Certificate has to be striped for some parsers
+        self.assertEqual(certificate,
+                         certificate.strip())
+
     def test_sign_with_invalid_csr(self):
         ca_key = self._generate_private_key()
         csr = 'test'
