@@ -13,8 +13,9 @@
 # under the License.
 
 import abc
-import mock
 from neutronclient.common import exceptions as n_exception
+from unittest import mock
+
 import six
 
 from magnum.common import exception
@@ -149,24 +150,25 @@ class TemplateDefinitionTestCase(base.TestCase):
         ]
 
         mock_stack = mock.MagicMock()
+        mock_cluster = mock.MagicMock()
         mock_stack.to_dict.return_value = {'outputs': heat_outputs}
 
         output = cmn_tdef.OutputMapping('key1')
-        value = output.get_output_value(mock_stack)
+        value = output.get_output_value(mock_stack, mock_cluster)
         self.assertEqual('value1', value)
 
         output = cmn_tdef.OutputMapping('key2')
-        value = output.get_output_value(mock_stack)
+        value = output.get_output_value(mock_stack, mock_cluster)
         self.assertEqual(["value2", "value3"], value)
 
         output = cmn_tdef.OutputMapping('key3')
-        value = output.get_output_value(mock_stack)
+        value = output.get_output_value(mock_stack, mock_cluster)
         self.assertIsNone(value)
 
         # verify stack with no 'outputs' attribute
         mock_stack.to_dict.return_value = {}
         output = cmn_tdef.OutputMapping('key1')
-        value = output.get_output_value(mock_stack)
+        value = output.get_output_value(mock_stack, mock_cluster)
         self.assertIsNone(value)
 
     def test_add_output_with_mapping_type(self):
@@ -184,13 +186,9 @@ class TemplateDefinitionTestCase(base.TestCase):
                       definition.output_mappings)
 
     def test_add_fip_env_lb_disabled_with_fp(self):
-        mock_cluster_template = mock.MagicMock(floating_ip_enabled=True,
-                                               master_lb_enabled=False,
-                                               labels={})
-        mock_cluster = mock.MagicMock(labels={})
+        mock_cluster = mock.MagicMock(master_lb_enabled=False, labels={})
         env_files = []
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
         self.assertEqual(
             [
                 cmn_tdef.COMMON_ENV_PATH + 'enable_floating_ip.yaml',
@@ -200,13 +198,11 @@ class TemplateDefinitionTestCase(base.TestCase):
         )
 
     def test_add_fip_env_lb_enabled_with_fp(self):
-        mock_cluster_template = mock.MagicMock(floating_ip_enabled=True,
-                                               master_lb_enabled=True,
-                                               labels={})
-        mock_cluster = mock.MagicMock(floating_ip_enabled=True, labels={})
+        mock_cluster = mock.MagicMock(floating_ip_enabled=True,
+                                      master_lb_enabled=True,
+                                      labels={})
         env_files = []
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
         self.assertEqual(
             [
                 cmn_tdef.COMMON_ENV_PATH + 'enable_floating_ip.yaml',
@@ -216,13 +212,9 @@ class TemplateDefinitionTestCase(base.TestCase):
         )
 
     def test_add_fip_env_lb_disabled_without_fp(self):
-        mock_cluster_template = mock.MagicMock(floating_ip_enabled=False,
-                                               master_lb_enabled=False,
-                                               labels={})
         mock_cluster = mock.MagicMock(labels={}, floating_ip_enabled=False)
         env_files = []
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
         self.assertEqual(
             [
                 cmn_tdef.COMMON_ENV_PATH + 'disable_floating_ip.yaml',
@@ -232,13 +224,9 @@ class TemplateDefinitionTestCase(base.TestCase):
         )
 
     def test_add_fip_env_lb_enabled_without_fp(self):
-        mock_cluster_template = mock.MagicMock(floating_ip_enabled=False,
-                                               master_lb_enabled=True,
-                                               labels={})
         mock_cluster = mock.MagicMock(labels={}, floating_ip_enabled=False,)
         env_files = []
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
         self.assertEqual(
             [
                 cmn_tdef.COMMON_ENV_PATH + 'disable_floating_ip.yaml',
@@ -248,17 +236,11 @@ class TemplateDefinitionTestCase(base.TestCase):
         )
 
     def test_add_fip_env_lb_fip_enabled_without_fp(self):
-        mock_cluster_template = mock.MagicMock(
-            floating_ip_enabled=False,
-            master_lb_enabled=True,
-            labels={"master_lb_floating_ip_enabled": "true"}
-        )
         mock_cluster = mock.MagicMock(
             labels={"master_lb_floating_ip_enabled": "true"},
             floating_ip_enabled=False,)
         env_files = []
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
         self.assertEqual(
             [
                 cmn_tdef.COMMON_ENV_PATH + 'disable_floating_ip.yaml',
@@ -268,18 +250,12 @@ class TemplateDefinitionTestCase(base.TestCase):
         )
 
     def test_add_fip_env_lb_enable_lbfip_disable(self):
-        mock_cluster_template = mock.MagicMock(
-            floating_ip_enabled=False,
-            master_lb_enabled=True,
-            labels={"master_lb_floating_ip_enabled": "false"}
-        )
         mock_cluster = mock.MagicMock(
             labels={"master_lb_floating_ip_enabled": "false"},
             floating_ip_enabled=False,)
         env_files = []
 
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
 
         self.assertEqual(
             [
@@ -290,18 +266,12 @@ class TemplateDefinitionTestCase(base.TestCase):
         )
 
     def test_add_fip_env_lb_enable_lbfip_template_disable_cluster_enable(self):
-        mock_cluster_template = mock.MagicMock(
-            floating_ip_enabled=False,
-            master_lb_enabled=True,
-            labels={}
-        )
         mock_cluster = mock.MagicMock(
             floating_ip_enabled=True,
             labels={})
         env_files = []
 
-        cmn_tdef.add_fip_env_file(env_files, mock_cluster_template,
-                                  mock_cluster)
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
 
         self.assertEqual(
             [
@@ -311,17 +281,21 @@ class TemplateDefinitionTestCase(base.TestCase):
             env_files
         )
 
-    @mock.patch('magnum.drivers.common.driver.Driver.get_driver')
-    def test_base_get_scale_params(self, mock_driver):
-        mock_context = mock.MagicMock()
-        mock_cluster = mock.MagicMock()
-        mock_driver.return_value = swarm_v2_dr.Driver()
-        cluster_driver = driver.Driver.get_driver('vm',
-                                                  'fedora-atomic',
-                                                  'swarm-mode')
-        definition = cluster_driver.get_template_definition()
-        self.assertEqual(definition.get_scale_params(mock_context,
-                                                     mock_cluster), {})
+    def test_add_fip_master_lb_fip_disabled_cluster_fip_enabled(self):
+        mock_cluster = mock.MagicMock(
+            labels={"master_lb_floating_ip_enabled": "false"},
+            floating_ip_enabled=True,)
+        env_files = []
+
+        cmn_tdef.add_fip_env_file(env_files, mock_cluster)
+
+        self.assertEqual(
+            [
+                cmn_tdef.COMMON_ENV_PATH + 'enable_floating_ip.yaml',
+                cmn_tdef.COMMON_ENV_PATH + 'enable_lb_floating_ip.yaml'
+            ],
+            env_files
+        )
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -369,6 +343,7 @@ class BaseK8sTemplateDefinitionTestCase(base.TestCase):
         mock_stack.to_dict.return_value = {'outputs': outputs}
         mock_cluster_template = mock.MagicMock()
         mock_cluster_template.floating_ip_enabled = floating_ip_enabled
+        self.mock_cluster.floating_ip_enabled = floating_ip_enabled
 
         definition.update_outputs(mock_stack, mock_cluster_template,
                                   self.mock_cluster)
@@ -400,16 +375,24 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
         mock_cluster = mock.MagicMock()
 
         removal_nodes = ['node1', 'node2']
+        node_count = 5
         mock_scale_manager = mock.MagicMock()
         mock_scale_manager.get_removal_nodes.return_value = removal_nodes
 
         definition = k8sa_tdef.AtomicK8sTemplateDefinition()
 
         scale_params = definition.get_scale_params(mock_context, mock_cluster,
+                                                   node_count,
                                                    mock_scale_manager)
-        expected_scale_params = {'minions_to_remove': ['node1', 'node2']}
+        expected_scale_params = {
+            'minions_to_remove': ['node1', 'node2'],
+            'number_of_minions': 5
+        }
         self.assertEqual(scale_params, expected_scale_params)
 
+    @mock.patch('magnum.common.neutron.get_subnet')
+    @mock.patch('magnum.drivers.heat.k8s_template_def.K8sTemplateDefinition'
+                '._set_master_lb_allowed_cidrs')
     @mock.patch('magnum.common.neutron.get_fixed_network_name')
     @mock.patch('magnum.common.keystone.is_octavia_enabled')
     @mock.patch('magnum.common.clients.OpenStackClients')
@@ -427,7 +410,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
                             mock_get_output, mock_get_params,
                             mock_get_discovery_url, mock_osc_class,
                             mock_enable_octavia,
-                            mock_get_fixed_network_name):
+                            mock_get_fixed_network_name,
+                            mock_set_master_lb_allowed_cidrs,
+                            mock_get_subnet):
         mock_generate_csr_and_key.return_value = {'csr': 'csr',
                                                   'private_key': 'private_key',
                                                   'public_key': 'public_key'}
@@ -441,6 +426,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
         mock_cluster_template.network_driver = 'flannel'
         external_network_id = '17e4e301-b7f3-4996-b3dd-97b3a700174b'
         mock_cluster_template.external_network_id = external_network_id
+        mock_cluster_template.no_proxy = ""
         mock_cluster = mock.MagicMock()
         fixed_network_name = 'fixed_network'
         mock_get_fixed_network_name.return_value = fixed_network_name
@@ -459,6 +445,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
 
         mock_context.auth_url = 'http://192.168.10.10:5000/v3'
         mock_context.user_name = 'fake_user'
+        mock_get_subnet.return_value = '20.200.0.0/16'
 
         flannel_cidr = mock_cluster.labels.get('flannel_network_cidr')
         flannel_subnet = mock_cluster.labels.get(
@@ -506,6 +493,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'calico_kube_controllers_tag')
         calico_ipv4pool = mock_cluster.labels.get(
             'calico_ipv4pool')
+        calico_ipv4pool_ipip = mock_cluster.labels.get(
+            'calico_ipv4pool_ipip')
         if mock_cluster_template.network_driver == 'flannel':
             pods_network_cidr = flannel_cidr
         elif mock_cluster_template.network_driver == 'calico':
@@ -565,6 +554,12 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'tiller_tag')
         tiller_namespace = mock_cluster.labels.get(
             'tiller_namespace')
+        helm_client_url = mock_cluster.labels.get(
+            'helm_client_url')
+        helm_client_sha256 = mock_cluster.labels.get(
+            'helm_client_sha256')
+        helm_client_tag = mock_cluster.labels.get(
+            'helm_client_tag')
         npd_tag = mock_cluster.labels.get('node_problem_detector_tag')
         traefik_ingress_controller_tag = mock_cluster.labels.get(
             'traefik_ingress_controller_tag')
@@ -576,6 +571,20 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'magnum_auto_healer_tag')
         auto_scaling_enabled = mock_cluster.labels.get(
             'auto_scaling_enabled')
+        cinder_csi_enabled = mock_cluster.labels.get(
+            'cinder_csi_enabled')
+        cinder_csi_plugin_tag = mock_cluster.labels.get(
+            'cinder_csi_plugin_tag')
+        csi_attacher_tag = mock_cluster.labels.get(
+            'csi_attacher_tag')
+        csi_provisioner_tag = mock_cluster.labels.get(
+            'csi_provisioner_tag')
+        csi_snapshotter_tag = mock_cluster.labels.get(
+            'csi_snapshotter_tag')
+        csi_resizer_tag = mock_cluster.labels.get(
+            'csi_resizer_tag')
+        csi_node_driver_registrar_tag = mock_cluster.labels.get(
+            'csi_node_driver_registrar_tag')
         draino_tag = mock_cluster.labels.get('draino_tag')
         autoscaler_tag = mock_cluster.labels.get('autoscaler_tag')
         min_node_count = mock_cluster.labels.get('min_node_count')
@@ -587,6 +596,17 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
         ostree_remote = mock_cluster.labels.get('ostree_remote')
         ostree_commit = mock_cluster.labels.get('ostree_commit')
         use_podman = mock_cluster.labels.get('use_podman')
+        selinux_mode = mock_cluster.labels.get('selinux_mode')
+        container_runtime = mock_cluster.labels.get('container_runtime')
+        containerd_version = mock_cluster.labels.get('containerd_version')
+        containerd_tarball_url = mock_cluster.labels.get(
+            'containerd_tarball_url')
+        containerd_tarball_sha256 = mock_cluster.labels.get(
+            'containerd_tarball_sha256')
+        kube_image_digest = mock_cluster.labels.get('kube_image_digest')
+        metrics_scraper_tag = mock_cluster.labels.get('metrics_scraper_tag')
+        master_lb_allowed_cidrs = mock_cluster.labels.get(
+            'master_lb_allowed_cidrs')
 
         k8s_def = k8sa_tdef.AtomicK8sTemplateDefinition()
 
@@ -635,6 +655,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'calico_tag': calico_tag,
             'calico_kube_controllers_tag': calico_kube_controllers_tag,
             'calico_ipv4pool': calico_ipv4pool,
+            'calico_ipv4pool_ipip': calico_ipv4pool_ipip,
             'cgroup_driver': cgroup_driver,
             'pods_network_cidr': pods_network_cidr,
             'ingress_controller': ingress_controller,
@@ -662,11 +683,21 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'tiller_enabled': tiller_enabled,
             'tiller_tag': tiller_tag,
             'tiller_namespace': tiller_namespace,
+            'helm_client_url': helm_client_url,
+            'helm_client_sha256': helm_client_sha256,
+            'helm_client_tag': helm_client_tag,
             'node_problem_detector_tag': npd_tag,
             'auto_healing_enabled': auto_healing_enabled,
             'auto_healing_controller': auto_healing_controller,
             'magnum_auto_healer_tag': magnum_auto_healer_tag,
             'auto_scaling_enabled': auto_scaling_enabled,
+            'cinder_csi_enabled': cinder_csi_enabled,
+            'cinder_csi_plugin_tag': cinder_csi_plugin_tag,
+            'csi_attacher_tag': csi_attacher_tag,
+            'csi_provisioner_tag': csi_provisioner_tag,
+            'csi_snapshotter_tag': csi_snapshotter_tag,
+            'csi_resizer_tag': csi_resizer_tag,
+            'csi_node_driver_registrar_tag': csi_node_driver_registrar_tag,
             'draino_tag': draino_tag,
             'autoscaler_tag': autoscaler_tag,
             'min_node_count': min_node_count,
@@ -682,6 +713,16 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'ostree_remote': ostree_remote,
             'ostree_commit': ostree_commit,
             'use_podman': use_podman,
+            'selinux_mode': selinux_mode,
+            'kube_image_digest': kube_image_digest,
+            'container_runtime': container_runtime,
+            'containerd_version': containerd_version,
+            'containerd_tarball_url': containerd_tarball_url,
+            'containerd_tarball_sha256': containerd_tarball_sha256,
+            'post_install_manifest_url': '',
+            'metrics_scraper_tag': metrics_scraper_tag,
+            'master_lb_allowed_cidrs': master_lb_allowed_cidrs,
+            'fixed_subnet_cidr': '20.200.0.0/16',
         }}
         mock_get_params.assert_called_once_with(mock_context,
                                                 mock_cluster_template,
@@ -708,6 +749,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             mock_cluster.fixed_network
         )
 
+    @mock.patch('magnum.common.neutron.get_subnet')
     @mock.patch('magnum.common.neutron.get_external_network_id')
     @mock.patch('magnum.common.keystone.is_octavia_enabled')
     @mock.patch('magnum.common.clients.OpenStackClients')
@@ -725,7 +767,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
                                                 mock_get_discovery_url,
                                                 mock_osc_class,
                                                 mock_enable_octavia,
-                                                mock_get_external_network_id):
+                                                mock_get_external_network_id,
+                                                mock_get_subnet):
         mock_generate_csr_and_key.return_value = {'csr': 'csr',
                                                   'private_key': 'private_key',
                                                   'public_key': 'public_key'}
@@ -769,6 +812,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             mock_cluster_template.external_network_id
         )
 
+    @mock.patch('magnum.common.neutron.get_subnet')
     @mock.patch('magnum.common.keystone.is_octavia_enabled')
     @mock.patch('magnum.common.clients.OpenStackClients')
     @mock.patch('magnum.drivers.k8s_fedora_atomic_v1.template_def'
@@ -784,7 +828,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
                                              mock_get_params,
                                              mock_get_discovery_url,
                                              mock_osc_class,
-                                             mock_enable_octavia):
+                                             mock_enable_octavia,
+                                             mock_get_subnet):
         mock_generate_csr_and_key.return_value = {'csr': 'csr',
                                                   'private_key': 'private_key',
                                                   'public_key': 'public_key'}
@@ -824,6 +869,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             mock_cluster,
         )
 
+    @mock.patch('magnum.common.neutron.get_subnet')
     @mock.patch('magnum.common.keystone.is_octavia_enabled')
     @mock.patch('magnum.common.clients.OpenStackClients')
     @mock.patch('magnum.drivers.k8s_fedora_atomic_v1.template_def'
@@ -839,7 +885,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
                                             mock_get_params,
                                             mock_get_discovery_url,
                                             mock_osc_class,
-                                            mock_enable_octavia):
+                                            mock_enable_octavia,
+                                            mock_get_subnet):
         mock_generate_csr_and_key.return_value = {'csr': 'csr',
                                                   'private_key': 'private_key',
                                                   'public_key': 'public_key'}
@@ -878,6 +925,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             actual_params.get("ingress_controller")
         )
 
+    @mock.patch('magnum.common.neutron.get_subnet')
+    @mock.patch('magnum.drivers.heat.k8s_template_def.K8sTemplateDefinition'
+                '._set_master_lb_allowed_cidrs')
     @mock.patch('magnum.common.keystone.is_octavia_enabled')
     @mock.patch('magnum.common.clients.OpenStackClients')
     @mock.patch('magnum.drivers.heat.template_def'
@@ -893,7 +943,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
                                      mock_sign_node_certificate,
                                      mock_get_output, mock_get_params,
                                      mock_get_discovery_url, mock_osc_class,
-                                     mock_enable_octavia):
+                                     mock_enable_octavia,
+                                     mock_set_master_lb_allowed_cidrs,
+                                     mock_get_subnet):
         mock_generate_csr_and_key.return_value = {'csr': 'csr',
                                                   'private_key': 'private_key',
                                                   'public_key': 'public_key'}
@@ -907,6 +959,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
         mock_cluster_template.network_driver = 'calico'
         external_network_id = '17e4e301-b7f3-4996-b3dd-97b3a700174b'
         mock_cluster_template.external_network_id = external_network_id
+        mock_cluster_template.no_proxy = ""
         mock_cluster = mock.MagicMock()
         fixed_network_name = 'fixed_network'
         mock_cluster.fixed_network = fixed_network_name
@@ -923,6 +976,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
 
         mock_context.auth_url = 'http://192.168.10.10:5000/v3'
         mock_context.user_name = 'fake_user'
+        mock_get_subnet.return_value = "20.200.0.0/16"
 
         flannel_cidr = mock_cluster.labels.get('flannel_network_cidr')
         flannel_subnet = mock_cluster.labels.get(
@@ -970,6 +1024,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'calico_kube_controllers_tag')
         calico_ipv4pool = mock_cluster.labels.get(
             'calico_ipv4pool')
+        calico_ipv4pool_ipip = mock_cluster.labels.get(
+            'calico_ipv4pool_ipip')
         if mock_cluster_template.network_driver == 'flannel':
             pods_network_cidr = flannel_cidr
         elif mock_cluster_template.network_driver == 'calico':
@@ -1029,6 +1085,12 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'tiller_tag')
         tiller_namespace = mock_cluster.labels.get(
             'tiller_namespace')
+        helm_client_url = mock_cluster.labels.get(
+            'helm_client_url')
+        helm_client_sha256 = mock_cluster.labels.get(
+            'helm_client_sha256')
+        helm_client_tag = mock_cluster.labels.get(
+            'helm_client_tag')
         npd_tag = mock_cluster.labels.get('node_problem_detector_tag')
         traefik_ingress_controller_tag = mock_cluster.labels.get(
             'traefik_ingress_controller_tag')
@@ -1040,6 +1102,20 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'magnum_auto_healer_tag')
         auto_scaling_enabled = mock_cluster.labels.get(
             'auto_scaling_enabled')
+        cinder_csi_enabled = mock_cluster.labels.get(
+            'cinder_csi_enabled')
+        cinder_csi_plugin_tag = mock_cluster.labels.get(
+            'cinder_csi_plugin_tag')
+        csi_attacher_tag = mock_cluster.labels.get(
+            'csi_attacher_tag')
+        csi_provisioner_tag = mock_cluster.labels.get(
+            'csi_provisioner_tag')
+        csi_snapshotter_tag = mock_cluster.labels.get(
+            'csi_snapshotter_tag')
+        csi_resizer_tag = mock_cluster.labels.get(
+            'csi_resizer_tag')
+        csi_node_driver_registrar_tag = mock_cluster.labels.get(
+            'csi_node_driver_registrar_tag')
         draino_tag = mock_cluster.labels.get('draino_tag')
         autoscaler_tag = mock_cluster.labels.get('autoscaler_tag')
         min_node_count = mock_cluster.labels.get('min_node_count')
@@ -1051,6 +1127,18 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
         ostree_remote = mock_cluster.labels.get('ostree_remote')
         ostree_commit = mock_cluster.labels.get('ostree_commit')
         use_podman = mock_cluster.labels.get('use_podman')
+        selinux_mode = mock_cluster.labels.get('selinux_mode')
+        container_runtime = mock_cluster.labels.get('container_runtime')
+        containerd_version = mock_cluster.labels.get('containerd_version')
+        containerd_tarball_url = mock_cluster.labels.get(
+            'containerd_tarball_url')
+        containerd_tarball_sha256 = mock_cluster.labels.get(
+            'containerd_tarball_sha256')
+        kube_image_digest = mock_cluster.labels.get('kube_image_digest')
+        metrics_scraper_tag = mock_cluster.labels.get('metrics_scraper_tag')
+
+        master_lb_allowed_cidrs = mock_cluster.labels.get(
+            'master_lb_allowed_cidrs')
 
         k8s_def = k8sa_tdef.AtomicK8sTemplateDefinition()
 
@@ -1101,6 +1189,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'calico_tag': calico_tag,
             'calico_kube_controllers_tag': calico_kube_controllers_tag,
             'calico_ipv4pool': calico_ipv4pool,
+            'calico_ipv4pool_ipip': calico_ipv4pool_ipip,
             'cgroup_driver': cgroup_driver,
             'pods_network_cidr': pods_network_cidr,
             'ingress_controller': ingress_controller,
@@ -1128,11 +1217,21 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'tiller_enabled': tiller_enabled,
             'tiller_tag': tiller_tag,
             'tiller_namespace': tiller_namespace,
+            'helm_client_url': helm_client_url,
+            'helm_client_sha256': helm_client_sha256,
+            'helm_client_tag': helm_client_tag,
             'node_problem_detector_tag': npd_tag,
             'auto_healing_enabled': auto_healing_enabled,
             'auto_healing_controller': auto_healing_controller,
             'magnum_auto_healer_tag': magnum_auto_healer_tag,
             'auto_scaling_enabled': auto_scaling_enabled,
+            'cinder_csi_enabled': cinder_csi_enabled,
+            'cinder_csi_plugin_tag': cinder_csi_plugin_tag,
+            'csi_attacher_tag': csi_attacher_tag,
+            'csi_provisioner_tag': csi_provisioner_tag,
+            'csi_snapshotter_tag': csi_snapshotter_tag,
+            'csi_resizer_tag': csi_resizer_tag,
+            'csi_node_driver_registrar_tag': csi_node_driver_registrar_tag,
             'draino_tag': draino_tag,
             'autoscaler_tag': autoscaler_tag,
             'min_node_count': min_node_count,
@@ -1148,6 +1247,16 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             'ostree_remote': ostree_remote,
             'ostree_commit': ostree_commit,
             'use_podman': use_podman,
+            'selinux_mode': selinux_mode,
+            'kube_image_digest': kube_image_digest,
+            'container_runtime': container_runtime,
+            'containerd_version': containerd_version,
+            'containerd_tarball_url': containerd_tarball_url,
+            'containerd_tarball_sha256': containerd_tarball_sha256,
+            'post_install_manifest_url': '',
+            'metrics_scraper_tag': metrics_scraper_tag,
+            'master_lb_allowed_cidrs': master_lb_allowed_cidrs,
+            'fixed_subnet_cidr': '20.200.0.0/16',
         }}
         mock_get_params.assert_called_once_with(mock_context,
                                                 mock_cluster_template,
@@ -1411,6 +1520,49 @@ class AtomicK8sTemplateDefinitionTestCase(BaseK8sTemplateDefinitionTestCase):
             is_master=False
         )
 
+    def test_set_master_lb_allowed_cidrs(self):
+        definition = self.get_definition()
+        extra_params = {"master_lb_allowed_cidrs": "192.168.0.0/16"}
+        mock_cluster = mock.MagicMock()
+        mock_context = mock.MagicMock()
+        mock_cluster.labels = {}
+
+        definition._set_master_lb_allowed_cidrs(mock_context,
+                                                mock_cluster, extra_params)
+
+        self.assertEqual(extra_params["master_lb_allowed_cidrs"],
+                         "192.168.0.0/16,10.0.0.0/24")
+
+    def test_set_master_lb_allowed_cidrs_fixed_subnet_cidr(self):
+        definition = self.get_definition()
+        extra_params = {"master_lb_allowed_cidrs": "192.168.0.0/16"}
+        mock_cluster = mock.MagicMock()
+        mock_context = mock.MagicMock()
+        mock_cluster.labels = {"fixed_subnet_cidr": "100.0.0.0/24"}
+
+        definition._set_master_lb_allowed_cidrs(mock_context,
+                                                mock_cluster, extra_params)
+
+        self.assertEqual(extra_params["master_lb_allowed_cidrs"],
+                         "192.168.0.0/16,100.0.0.0/24")
+
+    @mock.patch('magnum.common.neutron.get_subnet')
+    def test_set_master_lb_allowed_cidrs_find_subnet_cidr(self,
+                                                          mock_get_subnet):
+        definition = self.get_definition()
+        extra_params = {"master_lb_allowed_cidrs": "192.168.0.0/16",
+                        "fixed_subnet": "fake_subnet_id"}
+        mock_cluster = mock.MagicMock()
+        mock_context = mock.MagicMock()
+        mock_cluster.labels = {}
+        mock_get_subnet.return_value = "172.24.0.0/16"
+
+        definition._set_master_lb_allowed_cidrs(mock_context,
+                                                mock_cluster, extra_params)
+
+        self.assertEqual(extra_params["master_lb_allowed_cidrs"],
+                         "192.168.0.0/16,172.24.0.0/16")
+
 
 class FedoraK8sIronicTemplateDefinitionTestCase(base.TestCase):
 
@@ -1559,6 +1711,7 @@ class AtomicSwarmModeTemplateDefinitionTestCase(base.TestCase):
         mock_stack.to_dict.return_value = {'outputs': outputs}
         mock_cluster_template = mock.MagicMock()
         mock_cluster_template.floating_ip_enabled = floating_ip_enabled
+        self.mock_cluster.floating_ip_enabled = floating_ip_enabled
 
         definition.update_outputs(mock_stack, mock_cluster_template,
                                   self.mock_cluster)
@@ -1635,6 +1788,13 @@ class AtomicSwarmModeTemplateDefinitionTestCase(base.TestCase):
         heat_param = swarm_def.get_heat_param(cluster_attr='uuid')
         self.assertEqual('cluster_uuid', heat_param)
 
+    def test_swarm_get_scale_params(self):
+        mock_context = mock.MagicMock()
+        swarm_def = swarm_v2_tdef.AtomicSwarmTemplateDefinition()
+        self.assertEqual(
+            swarm_def.get_scale_params(mock_context, self.mock_cluster, 7),
+            {'number_of_nodes': 7})
+
     def test_update_outputs(self):
         swarm_def = swarm_v2_tdef.AtomicSwarmTemplateDefinition()
 
@@ -1969,14 +2129,19 @@ class UbuntuMesosTemplateDefinitionTestCase(base.TestCase):
         mock_cluster.uuid = '5d12f6fd-a196-4bf0-ae4c-1f639a523a52'
 
         removal_nodes = ['node1', 'node2']
+        node_count = 7
         mock_scale_manager = mock.MagicMock()
         mock_scale_manager.get_removal_nodes.return_value = removal_nodes
 
         mesos_def = mesos_tdef.UbuntuMesosTemplateDefinition()
 
-        scale_params = mesos_def.get_scale_params(mock_context, mock_cluster,
-                                                  mock_scale_manager)
-        expected_scale_params = {'slaves_to_remove': ['node1', 'node2']}
+        scale_params = mesos_def.get_scale_params(
+            mock_context,
+            mock_cluster,
+            node_count,
+            mock_scale_manager)
+        expected_scale_params = {'slaves_to_remove': ['node1', 'node2'],
+                                 'number_of_slaves': 7}
         self.assertEqual(scale_params, expected_scale_params)
 
     def test_mesos_get_heat_param(self):
