@@ -14,8 +14,8 @@
 
 import datetime
 import gettext
+from unittest import mock
 
-import mock
 from oslo_versionedobjects import exception as object_exception
 from oslo_versionedobjects import fields
 from oslo_versionedobjects import fixture
@@ -28,8 +28,7 @@ gettext.install('magnum')
 
 
 @base.MagnumObjectRegistry.register
-class MyObj(base.MagnumPersistentObject, base.MagnumObject,
-            base.MagnumObjectDictCompat):
+class MyObj(base.MagnumPersistentObject, base.MagnumObject):
     VERSION = '1.0'
 
     fields = {'foo': fields.IntegerField(),
@@ -152,8 +151,7 @@ class _TestObject(object):
 
     def test_load_in_base(self):
         @base.MagnumObjectRegistry.register_if(False)
-        class Foo(base.MagnumPersistentObject, base.MagnumObject,
-                  base.MagnumObjectDictCompat):
+        class Foo(base.MagnumPersistentObject, base.MagnumObject):
             fields = {'foobar': fields.IntegerField()}
         obj = Foo(self.context)
         # NOTE(danms): Can't use assertRaisesRegexp() because of py26
@@ -292,19 +290,15 @@ class _TestObject(object):
     def test_get(self):
         obj = MyObj(self.context, foo=1)
         # Foo has value, should not get the default
-        self.assertEqual(1, obj.get('foo', 2))
+        self.assertEqual(1, getattr(obj, 'foo', 2))
         # Foo has value, should return the value without error
-        self.assertEqual(1, obj.get('foo'))
-        # Bar is not loaded, so we should get the default
-        self.assertEqual('not-loaded', obj.get('bar', 'not-loaded'))
+        self.assertEqual(1, getattr(obj, 'foo'))
         # Bar without a default should lazy-load
-        self.assertEqual('loaded!', obj.get('bar'))
+        self.assertEqual('loaded!', getattr(obj, 'bar'))
         # Bar now has a default, but loaded value should be returned
-        self.assertEqual('loaded!', obj.get('bar', 'not-loaded'))
+        self.assertEqual('loaded!', getattr(obj, 'bar', 'not-loaded'))
         # Invalid attribute should raise AttributeError
-        self.assertRaises(AttributeError, obj.get, 'nothing')
-        # ...even with a default
-        self.assertRaises(AttributeError, obj.get, 'nothing', 3)
+        self.assertFalse(hasattr(obj, 'nothing'))
 
     def test_object_inheritance(self):
         base_fields = list(base.MagnumPersistentObject.fields.keys())
@@ -330,8 +324,7 @@ class _TestObject(object):
 
     def test_obj_fields(self):
         @base.MagnumObjectRegistry.register_if(False)
-        class TestObj(base.MagnumPersistentObject, base.MagnumObject,
-                      base.MagnumObjectDictCompat):
+        class TestObj(base.MagnumPersistentObject, base.MagnumObject):
             fields = {'foo': fields.IntegerField()}
             obj_extra_fields = ['bar']
 
@@ -360,15 +353,18 @@ class TestObject(test_base.TestCase, _TestObject):
 # object (fields and remotable methods). If either the version or hash
 # change, the static tree needs to be updated.
 # For more information on object version testing, read
-# http://docs.openstack.org/developer/magnum/objects.html
+# https://docs.openstack.org/magnum/latest/contributor/objects.html
 object_data = {
-    'Cluster': '1.9-f9838e23eef5f1a7d9606c1ccce21800',
-    'ClusterTemplate': '1.17-65a95ef932dd08800a83871eb3cf312b',
+    'Cluster': '1.22-39ae1aa9ed1e90ee05f67f64b5fce4bb',
+    'ClusterTemplate': '1.19-3b0b2b3933d0955abf3ab40111744960',
     'Certificate': '1.1-1924dc077daa844f0f9076332ef96815',
-    'MyObj': '1.0-b43567e512438205e32f4e95ca616697',
     'MyObj': '1.0-34c4b1aadefd177b13f9a2f894cc23cd',
     'X509KeyPair': '1.2-d81950af36c59a71365e33ce539d24f9',
     'MagnumService': '1.0-2d397ec59b0046bd5ec35cd3e06efeca',
+    'Stats': '1.0-73a1cd6e3c0294c932a66547faba216c',
+    'Quota': '1.0-94e100aebfa88f7d8428e007f2049c18',
+    'Federation': '1.0-166da281432b083f0e4b851336e12e20',
+    'NodeGroup': '1.0-8cb4544a28a49860d816158a7c3060b1'
 }
 
 
@@ -386,8 +382,8 @@ class TestObjectVersions(test_base.TestCase):
                          "changed. Make sure the versions of the objects has "
                          "been bumped, and update the hashes in the static "
                          "fingerprints tree (object_data). For more "
-                         "information, read http://docs.openstack.org/"
-                         "developer/magnum/objects.html.")
+                         "information, read https://docs.openstack.org/"
+                         "magnum/latest/contributor/objects.html")
 
 
 class TestObjectSerializer(test_base.TestCase):
@@ -433,7 +429,9 @@ class TestObjectSerializer(test_base.TestCase):
         primitive = obj.obj_to_primitive()
         result = ser.deserialize_entity(self.context, primitive)
         if backported_to is None:
-            self.assertFalse(mock_indirection_api.object_backport.called)
+            self.assertEqual(
+                False,
+                mock_indirection_api.object_backport.called)
         else:
             self.assertEqual('backported', result)
             mock_indirection_api.object_backport.assert_called_with(
